@@ -1,6 +1,6 @@
 package Image::Magick::Q16;
 
-#  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+#  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
 #  dedicated to making software imaging solutions freely available.
 #
 #  You may not use this file except in compliance with the License.  You may
@@ -43,12 +43,13 @@ require AutoLoader;
       ConfigureError FatalErrorException
     );
 
-$VERSION = '6.97';
+$VERSION = '6.9.10';
 
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
     # XS function.  If a constant is not found then control is passed
     # to the AUTOLOAD in AutoLoader.
+    no warnings;
 
     my $constname;
     ($constname = $AUTOLOAD) =~ s/.*:://;
