@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -225,9 +225,9 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
     Allocate histogram image.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
@@ -257,7 +257,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
     Initialize histogram count arrays.
   */
   channel=image_info->channel;
-  (void) ResetMagickMemory(histogram,0,length*sizeof(*histogram));
+  (void) memset(histogram,0,length*sizeof(*histogram));
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
@@ -340,7 +340,9 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   */
   histogram=(MagickPixelPacket *) RelinquishMagickMemory(histogram);
   option=GetImageOption(image_info,"histogram:unique-colors");
-  if ((option == (const char *) NULL) || (IsMagickTrue(option) != MagickFalse))
+  if ((IsHistogramImage(image,exception) != MagickFalse) ||
+      (IsMagickTrue(option) != MagickFalse) ||
+      (GetImageOption(image_info,"format") != (const char *) NULL))
     {
       FILE
         *file;
@@ -376,6 +378,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   */
   (void) CopyMagickString(histogram_image->filename,image_info->filename,
     MaxTextExtent);
+  (void) ResetImagePage(histogram_image,"0x0+0+0");
   write_info=CloneImageInfo(image_info);
   *write_info->magick='\0';
   (void) SetImageInfo(write_info,1,&image->exception);
@@ -383,11 +386,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
       (LocaleCompare(write_info->magick,"HISTOGRAM") == 0))
     (void) FormatLocaleString(histogram_image->filename,MaxTextExtent,
       "miff:%s",write_info->filename);
-  histogram_image->blob=(BlobInfo *) DetachBlob(histogram_image->blob);
-  histogram_image->blob=CloneBlobInfo(image->blob);
   status=WriteImage(write_info,histogram_image);
-  image->blob=(BlobInfo *) DetachBlob(image->blob);
-  image->blob=CloneBlobInfo(histogram_image->blob);
   histogram_image=DestroyImage(histogram_image);
   write_info=DestroyImageInfo(write_info);
   return(status);
