@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -43,6 +43,7 @@
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/cache.h"
+#include "magick/channel.h"
 #include "magick/color-private.h"
 #include "magick/colormap.h"
 #include "magick/colormap-private.h"
@@ -73,28 +74,6 @@
 /*
   ImageMagick Macintosh PICT Methods.
 */
-#define ReadPixmap(pixmap) \
-{ \
-  pixmap.version=(short) ReadBlobMSBShort(image); \
-  pixmap.pack_type=(short) ReadBlobMSBShort(image); \
-  pixmap.pack_size=ReadBlobMSBLong(image); \
-  pixmap.horizontal_resolution=1UL*ReadBlobMSBShort(image); \
-  (void) ReadBlobMSBShort(image); \
-  pixmap.vertical_resolution=1UL*ReadBlobMSBShort(image); \
-  (void) ReadBlobMSBShort(image); \
-  pixmap.pixel_type=(short) ReadBlobMSBShort(image); \
-  pixmap.bits_per_pixel=(short) ReadBlobMSBShort(image); \
-  pixmap.component_count=(short) ReadBlobMSBShort(image); \
-  pixmap.component_size=(short) ReadBlobMSBShort(image); \
-  pixmap.plane_bytes=ReadBlobMSBLong(image); \
-  pixmap.table=ReadBlobMSBLong(image); \
-  pixmap.reserved=ReadBlobMSBLong(image); \
-  if ((EOFBlob(image) != MagickFalse) || (pixmap.bits_per_pixel <= 0) || \
-      (pixmap.bits_per_pixel > 32) || (pixmap.component_count <= 0) || \
-      (pixmap.component_count > 4) || (pixmap.component_size <= 0)) \
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader"); \
-}
-
 typedef struct _PICTCode
 {
   const char
@@ -418,6 +397,9 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
 static unsigned char *DecodeImage(Image *blob,Image *image,
   size_t bytes_per_line,const unsigned int bits_per_pixel,size_t *extent)
 {
+  MagickBooleanType
+    status;
+
   MagickSizeType
     number_pixels;
 
@@ -472,11 +454,16 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
   if (pixels == (unsigned char *) NULL)
     return((unsigned char *) NULL);
   *extent=row_bytes*image->rows*sizeof(*pixels);
-  (void) ResetMagickMemory(pixels,0,*extent);
+  (void) memset(pixels,0,*extent);
   scanline=(unsigned char *) AcquireQuantumMemory(row_bytes,2*
     sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
-    return((unsigned char *) NULL);
+    {
+      pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+      return((unsigned char *) NULL);
+    }
+  (void) memset(scanline,0,2*row_bytes*sizeof(*scanline));
+  status=MagickTrue;
   if (bytes_per_line < 8)
     {
       /*
@@ -489,22 +476,20 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
         count=ReadBlob(blob,(size_t) number_pixels,scanline);
         if (count != (ssize_t) number_pixels)
           {
-            (void) ThrowMagickException(&image->exception,GetMagickModule(),
-              CorruptImageError,"UnableToUncompressImage","`%s'",
-              image->filename);
+            status=MagickFalse;
             break;
           }
         p=ExpandBuffer(scanline,&number_pixels,bits_per_pixel);
         if ((q+number_pixels) > (pixels+(*extent)))
           {
-            (void) ThrowMagickException(&image->exception,GetMagickModule(),
-              CorruptImageError,"UnableToUncompressImage","`%s'",
-              image->filename);
+            status=MagickFalse;
             break;
           }
-        (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+        (void) memcpy(q,p,(size_t) number_pixels);
       }
       scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+      if (status == MagickFalse)
+        pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       return(pixels);
     }
   /*
@@ -516,18 +501,16 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
     if (bytes_per_line > 200)
       scanline_length=ReadBlobMSBShort(blob);
     else
-      scanline_length=1UL*ReadBlobByte(blob);
-    if (scanline_length >= row_bytes)
+      scanline_length=(size_t) ReadBlobByte(blob);
+    if ((scanline_length >= row_bytes) || (scanline_length == 0))
       {
-        (void) ThrowMagickException(&image->exception,GetMagickModule(),
-          CorruptImageError,"UnableToUncompressImage","`%s'",image->filename);
+        status=MagickFalse;
         break;
       }
     count=ReadBlob(blob,scanline_length,scanline);
     if (count != (ssize_t) scanline_length)
       {
-        (void) ThrowMagickException(&image->exception,GetMagickModule(),
-          CorruptImageError,"UnableToUncompressImage","`%s'",image->filename);
+        status=MagickFalse;
         break;
       }
     for (j=0; j < (ssize_t) scanline_length; )
@@ -537,7 +520,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           number_pixels=length*bytes_per_pixel;
           p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
           if ((q-pixels+number_pixels) <= *extent)
-            (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+            (void) memcpy(q,p,(size_t) number_pixels);
           q+=number_pixels;
           j+=(ssize_t) (length*bytes_per_pixel+1);
         }
@@ -549,13 +532,15 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           for (i=0; i < (ssize_t) length; i++)
           {
             if ((q-pixels+number_pixels) <= *extent)
-              (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+              (void) memcpy(q,p,(size_t) number_pixels);
             q+=number_pixels;
           }
           j+=(ssize_t) bytes_per_pixel+1;
         }
   }
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  if (status == MagickFalse)
+    pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   return(pixels);
 }
 
@@ -620,7 +605,7 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
     Pack scanline.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(scanline != (unsigned char *) NULL);
@@ -789,14 +774,38 @@ static MagickBooleanType IsPICT(const unsigned char *magick,const size_t length)
 %
 */
 
+static MagickBooleanType ReadPixmap(Image *image,PICTPixmap *pixmap)
+{
+  pixmap->version=(short) ReadBlobMSBShort(image);
+  pixmap->pack_type=(short) ReadBlobMSBShort(image);
+  pixmap->pack_size=ReadBlobMSBLong(image);
+  pixmap->horizontal_resolution=1UL*ReadBlobMSBShort(image);
+  (void) ReadBlobMSBShort(image);
+  pixmap->vertical_resolution=1UL*ReadBlobMSBShort(image);
+  (void) ReadBlobMSBShort(image);
+  pixmap->pixel_type=(short) ReadBlobMSBShort(image);
+  pixmap->bits_per_pixel=(short) ReadBlobMSBShort(image);
+  pixmap->component_count=(short) ReadBlobMSBShort(image);
+  pixmap->component_size=(short) ReadBlobMSBShort(image);
+  pixmap->plane_bytes=ReadBlobMSBLong(image);
+  pixmap->table=ReadBlobMSBLong(image);
+  pixmap->reserved=ReadBlobMSBLong(image);
+  if ((EOFBlob(image) != MagickFalse) || (pixmap->bits_per_pixel <= 0) ||
+      (pixmap->bits_per_pixel > 32) || (pixmap->component_count <= 0) ||
+      (pixmap->component_count > 4) || (pixmap->component_size <= 0))
+    return(MagickFalse);
+  return(MagickTrue);
+}
+
 static MagickBooleanType ReadRectangle(Image *image,PICTRectangle *rectangle)
 {
   rectangle->top=(short) ReadBlobMSBShort(image);
   rectangle->left=(short) ReadBlobMSBShort(image);
   rectangle->bottom=(short) ReadBlobMSBShort(image);
   rectangle->right=(short) ReadBlobMSBShort(image);
-  if ((EOFBlob(image) != MagickFalse) || (rectangle->left > rectangle->right) ||
-      (rectangle->top > rectangle->bottom))
+  if ((EOFBlob(image) != MagickFalse) ||
+      ((rectangle->bottom-rectangle->top) <= 0) ||
+      ((rectangle->right-rectangle->left) <= 0))
     return(MagickFalse);
   return(MagickTrue);
 }
@@ -804,12 +813,25 @@ static MagickBooleanType ReadRectangle(Image *image,PICTRectangle *rectangle)
 static Image *ReadPICTImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
+#define ThrowPICTException(exception,message) \
+{ \
+  if (tile_image != (Image *) NULL) \
+    tile_image=DestroyImage(tile_image); \
+  if (read_info != (ImageInfo *) NULL) \
+    read_info=DestroyImageInfo(read_info); \
+  ThrowReaderException((exception),(message)); \
+}
+
   char
     geometry[MaxTextExtent],
     header_ole[4];
 
   Image
-    *image;
+    *image,
+    *tile_image;
+
+  ImageInfo
+    *read_info;
 
   IndexPacket
     index;
@@ -858,12 +880,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -874,6 +896,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   /*
     Read PICT header.
   */
+  read_info=(ImageInfo *) NULL;
+  tile_image=(Image *) NULL;
   pixmap.bits_per_pixel=0;
   pixmap.component_count=0;
   /*
@@ -890,31 +914,31 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         break;
   (void) ReadBlobMSBShort(image);  /* skip picture size */
   if (ReadRectangle(image,&frame) == MagickFalse)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   while ((c=ReadBlobByte(image)) == 0) ;
   if (c != 0x11)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  version=ReadBlobByte(image);
+    ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+  version=(ssize_t) ReadBlobByte(image);
   if (version == 2)
     {
       c=ReadBlobByte(image);
       if (c != 0xff)
-        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+        ThrowPICTException(CorruptImageError,"ImproperImageHeader");
     }
   else
     if (version != 1)
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   if ((frame.left < 0) || (frame.right < 0) || (frame.top < 0) ||
       (frame.bottom < 0) || (frame.left >= frame.right) ||
       (frame.top >= frame.bottom))
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   /*
     Create black canvas.
   */
   flags=0;
   image->depth=8;
-  image->columns=1UL*(frame.right-frame.left);
-  image->rows=1UL*(frame.bottom-frame.top);
+  image->columns=(unsigned int) (frame.right-frame.left);
+  image->rows=(unsigned int) (frame.bottom-frame.top);
   image->x_resolution=DefaultResolution;
   image->y_resolution=DefaultResolution;
   image->units=UndefinedResolution;
@@ -925,6 +949,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         return(GetFirstImageInList(image));
       }
   status=SetImageExtent(image,image->columns,image->rows);
+  if (status != MagickFalse)
+    status=ResetImagePixels(image,exception);
   if (status == MagickFalse)
     {
       InheritException(exception,&image->exception);
@@ -945,6 +971,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       code=ReadBlobMSBSignedShort(image);
     if (code < 0)
       break;
+    if (code == 0)
+      continue;
     if (code > 0xa1)
       {
         if (image->debug != MagickFalse)
@@ -971,18 +999,19 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             if (ReadRectangle(image,&frame) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if (((frame.left & 0x8000) != 0) || ((frame.top & 0x8000) != 0))
               break;
-            image->columns=1UL*(frame.right-frame.left);
-            image->rows=1UL*(frame.bottom-frame.top);
+            image->columns=(size_t) (frame.right-frame.left);
+            image->rows=(size_t) (frame.bottom-frame.top);
             status=SetImageExtent(image,image->columns,image->rows);
+            if (status != MagickFalse)
+              status=ResetImagePixels(image,&image->exception);
             if (status == MagickFalse)
               {
                 InheritException(exception,&image->exception);
                 return(DestroyImageList(image));
               }
-            (void) SetImageBackgroundColor(image);
             break;
           }
           case 0x12:
@@ -999,7 +1028,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             /*
               Skip pattern definition.
             */
-            pattern=1L*ReadBlobMSBShort(image);
+            pattern=(ssize_t) ReadBlobMSBShort(image);
             for (i=0; i < 8; i++)
               if (ReadBlobByte(image) == EOF)
                 break;
@@ -1011,22 +1040,23 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             if (pattern != 1)
-              ThrowReaderException(CorruptImageError,"UnknownPatternType");
+              ThrowPICTException(CorruptImageError,"UnknownPatternType");
             length=ReadBlobMSBShort(image);
             if (ReadRectangle(image,&frame) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-            ReadPixmap(pixmap);
-            image->depth=1UL*pixmap.component_size;
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+            if (ReadPixmap(image,&pixmap) == MagickFalse)
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+            image->depth=(size_t) pixmap.component_size;
             image->x_resolution=1.0*pixmap.horizontal_resolution;
             image->y_resolution=1.0*pixmap.vertical_resolution;
             image->units=PixelsPerInchResolution;
             (void) ReadBlobMSBLong(image);
-            flags=1L*ReadBlobMSBShort(image);
+            flags=(ssize_t) ReadBlobMSBShort(image);
             length=ReadBlobMSBShort(image);
             for (i=0; i <= (ssize_t) length; i++)
               (void) ReadBlobMSBLong(image);
-            width=1UL*(frame.bottom-frame.top);
-            height=1UL*(frame.right-frame.left);
+            width=(size_t) (frame.bottom-frame.top);
+            height=(size_t) (frame.right-frame.left);
             if (pixmap.bits_per_pixel <= 8)
               length&=0x7fff;
             if (pixmap.bits_per_pixel == 16)
@@ -1040,7 +1070,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                     break;
               }
             else
-              for (j=0; j < (int) height; j++)
+              for (i=0; i < (ssize_t) height; i++)
+              {
+                if (EOFBlob(image) != MagickFalse)
+                  break;
                 if (length > 200)
                   {
                     for (j=0; j < (ssize_t) ReadBlobMSBShort(image); j++)
@@ -1051,6 +1084,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   for (j=0; j < (ssize_t) ReadBlobByte(image); j++)
                     if (ReadBlobByte(image) == EOF)
                       break;
+              }
             break;
           }
           case 0x1b:
@@ -1107,15 +1141,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             unsigned char
               *pixels;
 
-            Image
-              *tile_image;
-
             /*
               Pixmap clipped by a rectangle.
             */
             bytes_per_line=0;
             if ((code != 0x9a) && (code != 0x9b))
-              bytes_per_line=1L*ReadBlobMSBShort(image);
+              bytes_per_line=(ssize_t) ReadBlobMSBShort(image);
             else
               {
                 (void) ReadBlobMSBShort(image);
@@ -1123,26 +1154,27 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 (void) ReadBlobMSBShort(image);
               }
             if (ReadRectangle(image,&frame) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             /*
               Initialize tile image.
             */
-            tile_image=CloneImage(image,1UL*(frame.right-frame.left),
-              1UL*(frame.bottom-frame.top),MagickTrue,exception);
+            tile_image=CloneImage(image,(size_t) (frame.right-frame.left),
+              (size_t) (frame.bottom-frame.top),MagickTrue,exception);
             if (tile_image == (Image *) NULL)
-              return((Image *) NULL);
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if ((code == 0x9a) || (code == 0x9b) ||
                 ((bytes_per_line & 0x8000) != 0))
               {
-                ReadPixmap(pixmap);
-                tile_image->depth=1UL*pixmap.component_size;
+                if (ReadPixmap(image,&pixmap) == MagickFalse)
+                  ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+                tile_image->depth=(size_t) pixmap.component_size;
                 tile_image->matte=pixmap.component_count == 4 ?
                   MagickTrue : MagickFalse;
                 tile_image->x_resolution=(double) pixmap.horizontal_resolution;
                 tile_image->y_resolution=(double) pixmap.vertical_resolution;
                 tile_image->units=PixelsPerInchResolution;
                 if (tile_image->matte != MagickFalse)
-                  image->matte=tile_image->matte;
+                  (void) SetImageAlphaChannel(tile_image,OpaqueAlphaChannel);
               }
             if ((code != 0x9a) && (code != 0x9b))
               {
@@ -1153,16 +1185,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if ((bytes_per_line & 0x8000) != 0)
                   {
                     (void) ReadBlobMSBLong(image);
-                    flags=1L*ReadBlobMSBShort(image);
+                    flags=(ssize_t) ReadBlobMSBShort(image);
                     tile_image->colors=1UL*ReadBlobMSBShort(image)+1;
                   }
                 status=AcquireImageColormap(tile_image,tile_image->colors);
                 if (status == MagickFalse)
-                  {
-                    tile_image=DestroyImage(tile_image);
-                    ThrowReaderException(ResourceLimitError,
-                      "MemoryAllocationFailed");
-                  }
+                  ThrowPICTException(ResourceLimitError,
+                    "MemoryAllocationFailed");
                 if ((bytes_per_line & 0x8000) != 0)
                   {
                     for (i=0; i < (ssize_t) tile_image->colors; i++)
@@ -1191,10 +1220,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                     }
                   }
               }
+            if (EOFBlob(image) != MagickFalse)
+              ThrowPICTException(CorruptImageError,
+                "InsufficientImageDataInFile");
             if (ReadRectangle(image,&source) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             if (ReadRectangle(image,&destination) == MagickFalse)
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             (void) ReadBlobMSBShort(image);
             if ((code == 0x91) || (code == 0x99) || (code == 0x9b))
               {
@@ -1208,16 +1240,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               }
             if ((code != 0x9a) && (code != 0x9b) &&
                 (bytes_per_line & 0x8000) == 0)
-              pixels=DecodeImage(image,tile_image,1UL*bytes_per_line,1,&extent);
+              pixels=DecodeImage(image,tile_image,(size_t) bytes_per_line,1,
+                &extent);
             else
-              pixels=DecodeImage(image,tile_image,1UL*bytes_per_line,1U*
-                pixmap.bits_per_pixel,&extent);
+              pixels=DecodeImage(image,tile_image,(unsigned int) bytes_per_line,
+                (unsigned int) pixmap.bits_per_pixel,&extent);
             if (pixels == (unsigned char *) NULL)
-              {
-                tile_image=DestroyImage(tile_image);
-                ThrowReaderException(ResourceLimitError,
-                  "MemoryAllocationFailed");
-              }
+              ThrowPICTException(CorruptImageError,"UnableToUncompressImage");
             /*
               Convert PICT tile image to pixel packets.
             */
@@ -1225,7 +1254,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             for (y=0; y < (ssize_t) tile_image->rows; y++)
             {
               if (p > (pixels+extent+image->columns))
-                ThrowReaderException(CorruptImageError,"NotEnoughPixelData");
+                {
+                  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+                  ThrowPICTException(CorruptImageError,"NotEnoughPixelData");
+                }
               q=QueueAuthenticPixels(tile_image,0,y,tile_image->columns,1,
                 exception);
               if (q == (PixelPacket *) NULL)
@@ -1248,7 +1280,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   {
                     if (pixmap.bits_per_pixel == 16)
                       {
-                        i=(*p++);
+                        i=(ssize_t) (*p++);
                         j=(*p);
                         SetPixelRed(q,ScaleCharToQuantum(
                           (unsigned char) ((i & 0x7c) << 1)));
@@ -1262,7 +1294,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       if (tile_image->matte == MagickFalse)
                         {
                           if (p > (pixels+extent+2*image->columns))
-                            ThrowReaderException(CorruptImageError,
+                            ThrowPICTException(CorruptImageError,
                               "NotEnoughPixelData");
                           SetPixelRed(q,ScaleCharToQuantum(*p));
                           SetPixelGreen(q,ScaleCharToQuantum(
@@ -1273,7 +1305,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       else
                         {
                           if (p > (pixels+extent+3*image->columns))
-                            ThrowReaderException(CorruptImageError,
+                            ThrowPICTException(CorruptImageError,
                               "NotEnoughPixelData");
                           SetPixelAlpha(q,ScaleCharToQuantum(*p));
                           SetPixelRed(q,ScaleCharToQuantum(
@@ -1296,16 +1328,17 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   if (p < pixels)
                     break;
                 }
-              status=SetImageProgress(image,LoadImageTag,y,tile_image->rows);
+              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+                tile_image->rows);
               if (status == MagickFalse)
                 break;
             }
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-            if (jpeg == MagickFalse)
+            if ((jpeg == MagickFalse) && (EOFBlob(image) == MagickFalse))
               if ((code == 0x9a) || (code == 0x9b) ||
                   ((bytes_per_line & 0x8000) != 0))
                 (void) CompositeImage(image,CopyCompositeOp,tile_image,
-                  destination.left,destination.top);
+                  (ssize_t) destination.left,(ssize_t) destination.top);
             tile_image=DestroyImage(tile_image);
             break;
           }
@@ -1325,7 +1358,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if (length == 0)
               break;
             (void) ReadBlobMSBLong(image);
-            length-=4;
+            length-=MagickMin(length,4);
             if (length == 0)
               break;
             info=(unsigned char *) AcquireQuantumMemory(length,sizeof(*info));
@@ -1333,7 +1366,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               break;
             count=ReadBlob(image,length,info);
             if (count != (ssize_t) length)
-              ThrowReaderException(ResourceLimitError,"UnableToReadImageData");
+              {
+                info=(unsigned char *) RelinquishMagickMemory(info);
+                ThrowPICTException(ResourceLimitError,"UnableToReadImageData");
+              }
             switch (type)
             {
               case 0xe0:
@@ -1343,8 +1379,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 status=SetImageProfile(image,"icc",profile);
                 profile=DestroyStringInfo(profile);
                 if (status == MagickFalse)
-                  ThrowReaderException(ResourceLimitError,
-                    "MemoryAllocationFailed");
+                  {
+                    info=(unsigned char *) RelinquishMagickMemory(info);
+                    ThrowPICTException(ResourceLimitError,
+                      "MemoryAllocationFailed");
+                  }
                 break;
               }
               case 0x1f2:
@@ -1353,8 +1392,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 SetStringInfoDatum(profile,info);
                 status=SetImageProfile(image,"iptc",profile);
                 if (status == MagickFalse)
-                  ThrowReaderException(ResourceLimitError,
-                    "MemoryAllocationFailed");
+                  {
+                    info=(unsigned char *) RelinquishMagickMemory(info);
+                    ThrowPICTException(ResourceLimitError,
+                      "MemoryAllocationFailed");
+                  }
                 profile=DestroyStringInfo(profile);
                 break;
               }
@@ -1393,15 +1435,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       continue;
     if (code == 0x8200)
       {
+        char
+          filename[MaxTextExtent];
+
         FILE
           *file;
 
-        Image
-          *tile_image;
-
-        ImageInfo
-          *read_info;
-
         int
           unique_file;
 
@@ -1412,7 +1451,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         read_info=CloneImageInfo(image_info);
         SetImageInfoBlob(read_info,(void *) NULL,0);
         file=(FILE *) NULL;
-        unique_file=AcquireUniqueFileResource(read_info->filename);
+        unique_file=AcquireUniqueFileResource(filename);
+        (void) FormatLocaleString(read_info->filename,MaxTextExtent,"jpeg:%s",
+          filename);
         if (unique_file != -1)
           file=fdopen(unique_file,"wb");
         if ((unique_file == -1) || (file == (FILE *) NULL))
@@ -1420,10 +1461,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             (void) RelinquishUniqueFileResource(read_info->filename);
             (void) CopyMagickString(image->filename,read_info->filename,
               MaxTextExtent);
-            ThrowFileException(exception,FileOpenError,
-              "UnableToCreateTemporaryFile",image->filename);
-            image=DestroyImageList(image);
-            return((Image *) NULL);
+            ThrowPICTException(FileOpenError,"UnableToCreateTemporaryFile");
           }
         length=ReadBlobMSBLong(image);
         if (length > 154)
@@ -1434,7 +1472,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               {
                 (void) fclose(file);
                 (void) RelinquishUniqueFileResource(read_info->filename);
-                ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+                ThrowPICTException(CorruptImageError,"ImproperImageHeader");
               }
             for (i=0; i < 122; i++)
               if (ReadBlobByte(image) == EOF)
@@ -1444,13 +1482,14 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               c=ReadBlobByte(image);
               if (c == EOF)
                 break;
-              (void) fputc(c,file);
+              if (fputc(c,file) != c)
+                break;
             }
           }
         (void) fclose(file);
         (void) close(unique_file);
         tile_image=ReadImage(read_info,exception);
-        (void) RelinquishUniqueFileResource(read_info->filename);
+        (void) RelinquishUniqueFileResource(filename);
         read_info=DestroyImageInfo(read_info);
         if (tile_image == (Image *) NULL)
           continue;
@@ -1461,8 +1500,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           MagickMax(image->columns,tile_image->columns),
           MagickMax(image->rows,tile_image->rows));
         (void) TransformImageColorspace(image,tile_image->colorspace);
-        (void) CompositeImage(image,CopyCompositeOp,tile_image,frame.left,
-          frame.right);
+        (void) CompositeImage(image,CopyCompositeOp,tile_image,(ssize_t)
+          frame.left,(ssize_t) frame.right);
         image->compression=tile_image->compression;
         tile_image=DestroyImage(tile_image);
         continue;
@@ -1668,9 +1707,9 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->columns > 65535L) || (image->rows > 65535L))
@@ -1737,13 +1776,22 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   if ((buffer == (unsigned char *) NULL) ||
       (packed_scanline == (unsigned char *) NULL) ||
       (scanline == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(scanline,0,row_bytes);
-  (void) ResetMagickMemory(packed_scanline,0,(size_t) (row_bytes+MaxCount));
+    {
+      if (scanline != (unsigned char *) NULL)
+        scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+      if (packed_scanline != (unsigned char *) NULL)
+        packed_scanline=(unsigned char *) RelinquishMagickMemory(
+          packed_scanline);
+      if (buffer != (unsigned char *) NULL)
+        buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    }
+  (void) memset(scanline,0,row_bytes);
+  (void) memset(packed_scanline,0,(size_t) (row_bytes+MaxCount));
   /*
     Write header, header size, size bounding box, version, and reserved.
   */
-  (void) ResetMagickMemory(buffer,0,PictInfoSize);
+  (void) memset(buffer,0,PictInfoSize);
   (void) WriteBlob(image,PictInfoSize,buffer);
   (void) WriteBlobMSBShort(image,0);
   (void) WriteBlobMSBShort(image,(unsigned short) size_rectangle.top);
@@ -1753,7 +1801,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   (void) WriteBlobMSBShort(image,PictVersion);
   (void) WriteBlobMSBShort(image,0x02ff);  /* version #2 */
   (void) WriteBlobMSBShort(image,PictInfoOp);
-  (void) WriteBlobMSBLong(image,0xFFFE0000UL);
+  (void) WriteBlobMSBLong(image,0xFFFE0000U);
   /*
     Write full size of the file, resolution, frame bounding box, and reserved.
   */
@@ -1784,13 +1832,13 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBShort(image,0xe0);
       (void) WriteBlobMSBShort(image,(unsigned short)
         (GetStringInfoLength(profile)+4));
-      (void) WriteBlobMSBLong(image,0x00000000UL);
+      (void) WriteBlobMSBLong(image,0x00000000U);
       (void) WriteBlob(image,GetStringInfoLength(profile),
         GetStringInfoDatum(profile));
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
       (void) WriteBlobMSBShort(image,4);
-      (void) WriteBlobMSBLong(image,0x00000002UL);
+      (void) WriteBlobMSBLong(image,0x00000002U);
     }
   /*
     Write crop region opcode and crop bounding box.
@@ -1833,50 +1881,51 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBShort(image,PictJPEGOp);
       (void) WriteBlobMSBLong(image,(unsigned int) length+154);
       (void) WriteBlobMSBShort(image,0x0000);
-      (void) WriteBlobMSBLong(image,0x00010000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00010000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x40000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00400000UL);
+      (void) WriteBlobMSBLong(image,0x00010000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00010000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x40000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00400000U);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,(unsigned short) image->rows);
       (void) WriteBlobMSBShort(image,(unsigned short) image->columns);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,768);
       (void) WriteBlobMSBShort(image,0x0000);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00566A70UL);
-      (void) WriteBlobMSBLong(image,0x65670000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000001UL);
-      (void) WriteBlobMSBLong(image,0x00016170UL);
-      (void) WriteBlobMSBLong(image,0x706C0000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00566A70U);
+      (void) WriteBlobMSBLong(image,0x65670000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000001U);
+      (void) WriteBlobMSBLong(image,0x00016170U);
+      (void) WriteBlobMSBLong(image,0x706C0000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
       (void) WriteBlobMSBShort(image,768);
       (void) WriteBlobMSBShort(image,(unsigned short) image->columns);
       (void) WriteBlobMSBShort(image,(unsigned short) image->rows);
       (void) WriteBlobMSBShort(image,(unsigned short) x_resolution);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,(unsigned short) y_resolution);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x87AC0001UL);
-      (void) WriteBlobMSBLong(image,0x0B466F74UL);
-      (void) WriteBlobMSBLong(image,0x6F202D20UL);
-      (void) WriteBlobMSBLong(image,0x4A504547UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x00000000UL);
-      (void) WriteBlobMSBLong(image,0x0018FFFFUL);
+      (void) WriteBlobMSBShort(image,0x0000);
+      (void) WriteBlobMSBLong(image,length);
+      (void) WriteBlobMSBShort(image,0x0001);
+      (void) WriteBlobMSBLong(image,0x0B466F74U);
+      (void) WriteBlobMSBLong(image,0x6F202D20U);
+      (void) WriteBlobMSBLong(image,0x4A504547U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x00000000U);
+      (void) WriteBlobMSBLong(image,0x0018FFFFU);
       (void) WriteBlob(image,length,blob);
       if ((length & 0x01) != 0)
         (void) WriteBlobByte(image,'\0');
@@ -1890,7 +1939,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   else
     {
       (void) WriteBlobMSBShort(image,PictPixmapOp);
-      (void) WriteBlobMSBLong(image,(size_t) base_address);
+      (void) WriteBlobMSBLong(image,(unsigned int) base_address);
     }
   (void) WriteBlobMSBShort(image,(unsigned short) (row_bytes | 0x8000));
   (void) WriteBlobMSBShort(image,(unsigned short) bounds.top);
@@ -1974,7 +2023,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   else
     if (image_info->compression == JPEGCompression)
       {
-        (void) ResetMagickMemory(scanline,0,row_bytes);
+        (void) memset(scanline,0,row_bytes);
         for (y=0; y < (ssize_t) image->rows; y++)
           count+=EncodeImage(image,scanline,(size_t) (row_bytes & 0x7FFF),
             packed_scanline);
