@@ -17,13 +17,13 @@
 %                                 March 2011                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -57,6 +57,7 @@
 #include "magick/memory_.h"
 #include "magick/option.h"
 #include "magick/pixel-accessor.h"
+#include "magick/profile.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -68,6 +69,9 @@
 #if defined(MAGICKCORE_WEBP_DELEGATE)
 #include <webp/decode.h>
 #include <webp/encode.h>
+#if defined(MAGICKCORE_WEBPMUX_DELEGATE)
+#include <webp/mux.h>
+#endif
 #endif
 
 /*
@@ -177,13 +181,15 @@ static MagickBooleanType IsWEBPImageLossless(const unsigned char *stream,
   /*
     Read simple header.
   */
+  if (length <= VP8_CHUNK_INDEX)
+    return(MagickFalse);
   if (stream[VP8_CHUNK_INDEX] != EXTENDED_HEADER)
     return(stream[VP8_CHUNK_INDEX] == LOSSLESS_FLAG ? MagickTrue : MagickFalse);
   /*
     Read extended header.
   */
   offset=RIFF_HEADER_SIZE+TAG_SIZE+CHUNK_SIZE_BYTES+VP8X_CHUNK_SIZE;
-  while (offset <= (ssize_t) length)
+  while (offset+TAG_SIZE <= (ssize_t) (length-TAG_SIZE))
   {
     uint32_t
       chunk_size,
@@ -204,6 +210,15 @@ static MagickBooleanType IsWEBPImageLossless(const unsigned char *stream,
 static Image *ReadWEBPImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
+#define ThrowWEBPException(severity,tag) \
+{ \
+  if (stream != (unsigned char *) NULL) \
+    stream=(unsigned char*) RelinquishMagickMemory(stream); \
+  if (webp_image != (WebPDecBuffer *) NULL) \
+    WebPFreeDecBuffer(webp_image); \
+  ThrowReaderException(severity,tag); \
+}
+
   Image
     *image;
 
@@ -240,12 +255,12 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -253,25 +268,28 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  stream=(unsigned char *) NULL;
   if (WebPInitDecoderConfig(&configure) == 0)
     ThrowReaderException(ResourceLimitError,"UnableToDecodeImageFile");
   webp_image->colorspace=MODE_RGBA;
   count=ReadBlob(image,12,header);
   if (count != 12)
-    ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+    ThrowWEBPException(CorruptImageError,"InsufficientImageDataInFile");
   status=IsWEBP(header,count);
   if (status == MagickFalse)
-    ThrowReaderException(CorruptImageError,"CorruptImage");
+    ThrowWEBPException(CorruptImageError,"CorruptImage");
   length=(size_t) (ReadWebPLSBWord(header+4)+8);
   if (length < 12)
-    ThrowReaderException(CorruptImageError,"CorruptImage");
+    ThrowWEBPException(CorruptImageError,"CorruptImage");
+  if (length > GetBlobSize(image))
+    ThrowWEBPException(CorruptImageError,"InsufficientImageDataInFile");
   stream=(unsigned char *) AcquireQuantumMemory(length,sizeof(*stream));
   if (stream == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    ThrowWEBPException(ResourceLimitError,"MemoryAllocationFailed");
   (void) memcpy(stream,header,12);
   count=ReadBlob(image,length-12,stream+12);
   if (count != (ssize_t) (length-12))
-    ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+    ThrowWEBPException(CorruptImageError,"InsufficientImageDataInFile");
   webp_status=WebPGetFeatures(stream,length,features);
   if (webp_status == VP8_STATUS_OK)
     {
@@ -279,8 +297,6 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
       image->rows=(size_t) features->height;
       image->depth=8;
       image->matte=features->has_alpha != 0 ? MagickTrue : MagickFalse;
-      if (IsWEBPImageLossless(stream,length) != MagickFalse)
-        image->quality=100;
       if (image_info->ping != MagickFalse)
         {
           stream=(unsigned char*) RelinquishMagickMemory(stream);
@@ -290,54 +306,55 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
       status=SetImageExtent(image,image->columns,image->rows);
       if (status == MagickFalse)
         {
+          stream=(unsigned char*) RelinquishMagickMemory(stream);
           InheritException(exception,&image->exception);
+          (void) CloseBlob(image);
           return(DestroyImageList(image));
         }
+      if (IsWEBPImageLossless(stream,length) != MagickFalse)
+        image->quality=100;
       webp_status=WebPDecode(stream,length,&configure);
     }
   if (webp_status != VP8_STATUS_OK)
+    switch (webp_status)
     {
-      stream=(unsigned char*) RelinquishMagickMemory(stream);
-      switch (webp_status)
+      case VP8_STATUS_OUT_OF_MEMORY:
       {
-        case VP8_STATUS_OUT_OF_MEMORY:
-        {
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-          break;
-        }
-        case VP8_STATUS_INVALID_PARAM:
-        {
-          ThrowReaderException(CorruptImageError,"invalid parameter");
-          break;
-        }
-        case VP8_STATUS_BITSTREAM_ERROR:
-        {
-          ThrowReaderException(CorruptImageError,"CorruptImage");
-          break;
-        }
-        case VP8_STATUS_UNSUPPORTED_FEATURE:
-        {
-          ThrowReaderException(CoderError,"DataEncodingSchemeIsNotSupported");
-          break;
-        }
-        case VP8_STATUS_SUSPENDED:
-        {
-          ThrowReaderException(CorruptImageError,"decoder suspended");
-          break;
-        }
-        case VP8_STATUS_USER_ABORT:
-        {
-          ThrowReaderException(CorruptImageError,"user abort");
-          break;
-        }
-        case VP8_STATUS_NOT_ENOUGH_DATA:
-        {
-          ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
-          break;
-        }
-        default:
-          ThrowReaderException(CorruptImageError,"CorruptImage");
+        ThrowWEBPException(ResourceLimitError,"MemoryAllocationFailed");
+        break;
       }
+      case VP8_STATUS_INVALID_PARAM:
+      {
+        ThrowWEBPException(CorruptImageError,"invalid parameter");
+        break;
+      }
+      case VP8_STATUS_BITSTREAM_ERROR:
+      {
+        ThrowWEBPException(CorruptImageError,"CorruptImage");
+        break;
+      }
+      case VP8_STATUS_UNSUPPORTED_FEATURE:
+      {
+        ThrowWEBPException(CoderError,"DataEncodingSchemeIsNotSupported");
+        break;
+      }
+      case VP8_STATUS_SUSPENDED:
+      {
+        ThrowWEBPException(CorruptImageError,"decoder suspended");
+        break;
+      }
+      case VP8_STATUS_USER_ABORT:
+      {
+        ThrowWEBPException(CorruptImageError,"user abort");
+        break;
+      }
+      case VP8_STATUS_NOT_ENOUGH_DATA:
+      {
+        ThrowWEBPException(CorruptImageError,"InsufficientImageDataInFile");
+        break;
+      }
+      default:
+        ThrowWEBPException(CorruptImageError,"CorruptImage");
     }
   p=(unsigned char *) webp_image->u.RGBA.rgba;
   for (y=0; y < (ssize_t) image->rows; y++)
@@ -367,7 +384,62 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
       break;
   }
   WebPFreeDecBuffer(webp_image);
+#if defined(MAGICKCORE_WEBPMUX_DELEGATE)
+  {
+    StringInfo
+      *profile;
+
+    uint32_t
+      webp_flags = 0;
+
+    WebPData
+     chunk,
+     content = { stream, length };
+
+    WebPMux
+      *mux;
+
+    /*
+      Extract any profiles.
+    */
+    mux=WebPMuxCreate(&content,0);
+    (void) memset(&chunk,0,sizeof(chunk));
+    WebPMuxGetFeatures(mux,&webp_flags);
+    if (webp_flags & ICCP_FLAG)
+      {
+        WebPMuxGetChunk(mux,"ICCP",&chunk);
+        profile=BlobToStringInfo(chunk.bytes,chunk.size);
+        if (profile != (StringInfo *) NULL)
+          {
+            SetImageProfile(image,"ICC",profile);
+            profile=DestroyStringInfo(profile);
+          }
+      }
+    if (webp_flags & EXIF_FLAG)
+      {
+        WebPMuxGetChunk(mux,"EXIF",&chunk);
+        profile=BlobToStringInfo(chunk.bytes,chunk.size);
+        if (profile != (StringInfo *) NULL)
+          {
+            SetImageProfile(image,"EXIF",profile);
+            profile=DestroyStringInfo(profile);
+          }
+      }
+    if (webp_flags & XMP_FLAG)
+      {
+        WebPMuxGetChunk(mux,"XMP",&chunk);
+        profile=BlobToStringInfo(chunk.bytes,chunk.size);
+        if (profile != (StringInfo *) NULL)
+          {
+            SetImageProfile(image,"XMP",profile);
+            profile=DestroyStringInfo(profile);
+          }
+      }
+    WebPMuxDelete(mux);
+  }
+#endif
   stream=(unsigned char*) RelinquishMagickMemory(stream);
+  (void) CloseBlob(image);
   return(image);
 }
 #endif
@@ -415,6 +487,7 @@ ModuleExport size_t RegisterWEBPImage(void)
 #endif
   entry->description=ConstantString("WebP Image Format");
   entry->mime_type=ConstantString("image/webp");
+  entry->seekable_stream=MagickTrue;
   entry->adjoin=MagickFalse;
   entry->module=ConstantString("WEBP");
   entry->magick=(IsImageFormatHandler *) IsWEBP;
@@ -487,12 +560,13 @@ static int WebPEncodeProgress(int percent,const WebPPicture* picture)
   MagickBooleanType
     status;
 
-  image=(Image *) picture->custom_ptr;
+  image=(Image *) picture->user_data;
   status=SetImageProgress(image,EncodeImageTag,percent-1,100);
   return(status == MagickFalse ? 0 : 1);
 }
 #endif
 
+#if !defined(MAGICKCORE_WEBPMUX_DELEGATE)
 static int WebPEncodeWriter(const unsigned char *stream,size_t length,
   const WebPPicture *const picture)
 {
@@ -500,8 +574,9 @@ static int WebPEncodeWriter(const unsigned char *stream,size_t length,
     *image;
 
   image=(Image *) picture->custom_ptr;
-  return(length != 0 ? (int) WriteBlob(image,length,stream) : 1);
+  return(length != 0 ? (WriteBlob(image,length,stream) == (ssize_t) length) : 1);
 }
+#endif
 
 static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
   Image *image)
@@ -524,22 +599,27 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
   ssize_t
     y;
 
+  WebPAuxStats
+    statistics;
+
   WebPConfig
     configure;
 
+#if defined(MAGICKCORE_WEBPMUX_DELEGATE)
+  WebPMemoryWriter
+    writer_info;
+#endif
+
   WebPPicture
     picture;
 
-  WebPAuxStats
-    statistics;
-
   /*
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->columns > 16383UL) || (image->rows > 16383UL))
@@ -549,10 +629,17 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
     return(status);
   if ((WebPPictureInit(&picture) == 0) || (WebPConfigInit(&configure) == 0))
     ThrowWriterException(ResourceLimitError,"UnableToEncodeImageFile");
+#if !defined(MAGICKCORE_WEBPMUX_DELEGATE)
   picture.writer=WebPEncodeWriter;
   picture.custom_ptr=(void *) image;
+#else
+  WebPMemoryWriterInit(&writer_info);
+  picture.writer=WebPMemoryWrite;
+  picture.custom_ptr=(&writer_info);
+#endif
 #if WEBP_DECODER_ABI_VERSION >= 0x0100
   picture.progress_hook=WebPEncodeProgress;
+  picture.user_data=(void *) image;
 #endif
   picture.stats=(&statistics);
   picture.width=(int) image->columns;
@@ -674,11 +761,11 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      *q++=(uint32_t) (image->matte != MagickFalse ?
+      *q++=(uint32_t) (image->matte != MagickFalse ? (uint32_t)
         ScaleQuantumToChar(GetPixelAlpha(p)) << 24 : 0xff000000u) |
-        (ScaleQuantumToChar(GetPixelRed(p)) << 16) |
-        (ScaleQuantumToChar(GetPixelGreen(p)) << 8) |
-        (ScaleQuantumToChar(GetPixelBlue(p)));
+        ((uint32_t) ScaleQuantumToChar(GetPixelRed(p)) << 16) |
+        ((uint32_t) ScaleQuantumToChar(GetPixelGreen(p)) << 8) |
+        ((uint32_t) ScaleQuantumToChar(GetPixelBlue(p)));
       p++;
     }
     status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
@@ -755,8 +842,74 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
         CorruptImageError,(char *) message,"`%s'",image->filename);
     }
+#if defined(MAGICKCORE_WEBPMUX_DELEGATE)
+  {
+    const StringInfo
+      *profile;
+
+    WebPData
+      chunk,
+      image_chunk = { writer_info.mem, writer_info.size };
+
+    WebPMux
+      *mux;
+
+    WebPMuxError
+      mux_error;
+
+    /*
+      Set image profiles (if any).
+    */
+    mux_error=WEBP_MUX_OK;
+    (void) memset(&chunk,0,sizeof(chunk));
+    mux=WebPMuxNew();
+    profile=GetImageProfile(image,"ICC");
+    if ((profile != (StringInfo *) NULL) && (mux_error == WEBP_MUX_OK))
+      {
+        chunk.bytes=GetStringInfoDatum(profile);
+        chunk.size=GetStringInfoLength(profile);
+        mux_error=WebPMuxSetChunk(mux,"ICCP",&chunk,0);
+      }
+    profile=GetImageProfile(image,"EXIF");
+    if ((profile != (StringInfo *) NULL) && (mux_error == WEBP_MUX_OK))
+      {
+        chunk.bytes=GetStringInfoDatum(profile);
+        chunk.size=GetStringInfoLength(profile);
+        mux_error=WebPMuxSetChunk(mux,"EXIF",&chunk,0);
+      }
+    profile=GetImageProfile(image,"XMP");
+    if ((profile != (StringInfo *) NULL) && (mux_error == WEBP_MUX_OK))
+      {
+        chunk.bytes=GetStringInfoDatum(profile);
+        chunk.size=GetStringInfoLength(profile);
+        mux_error=WebPMuxSetChunk(mux,"XMP",&chunk,0);
+      }
+    if (mux_error != WEBP_MUX_OK)
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        ResourceLimitError,"UnableToEncodeImageFile","`%s'",image->filename);
+    if (chunk.size != 0)
+      {
+        WebPData
+          picture_profiles = { writer_info.mem, writer_info.size };
+
+        /*
+          Replace original container with image profile (if any).
+        */
+        WebPMuxSetImage(mux,&image_chunk,1);
+        mux_error=WebPMuxAssemble(mux,&picture_profiles);
+        WebPMemoryWriterClear(&writer_info);
+        writer_info.size=picture_profiles.size;
+        writer_info.mem=(unsigned char *) picture_profiles.bytes;
+      }
+    WebPMuxDelete(mux);
+  }
+  (void) WriteBlob(image,writer_info.size,writer_info.mem);
+#endif
   picture.argb=(uint32_t *) NULL;
   WebPPictureFree(&picture);
+#if defined(MAGICKCORE_WEBPMUX_DELEGATE)
+  WebPMemoryWriterClear(&writer_info);
+#endif
   pixel_info=RelinquishVirtualMemory(pixel_info);
   (void) CloseBlob(image);
   return(webp_status == 0 ? MagickFalse : MagickTrue);
