@@ -17,13 +17,13 @@
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -117,11 +117,11 @@ MagickExport Image *CombineImages(const Image *image,const ChannelType channel,
     Ensure the image are the same size.
   */
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
   {
     if ((next->columns != image->columns) || (next->rows != image->rows))
@@ -137,7 +137,12 @@ MagickExport Image *CombineImages(const Image *image,const ChannelType channel,
       return((Image *) NULL);
     }
   if (IssRGBCompatibleColorspace(image->colorspace) != MagickFalse)
-    (void) SetImageColorspace(combine_image,sRGBColorspace);
+    {
+      if (fabs(image->gamma-1.0) <= MagickEpsilon)
+        (void) SetImageColorspace(combine_image,RGBColorspace);
+      else
+        (void) SetImageColorspace(combine_image,sRGBColorspace);
+    }
   if ((channel & OpacityChannel) != 0)
     combine_image->matte=MagickTrue;
   (void) SetImageBackgroundColor(combine_image);
@@ -310,7 +315,7 @@ MagickExport MagickBooleanType GetImageAlphaChannel(const Image *image)
   assert(image != (const Image *) NULL);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   return(image->matte);
 }
 
@@ -357,11 +362,11 @@ MagickExport Image *SeparateImage(const Image *image,const ChannelType channel,
     Initialize separate image attributes.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   separate_image=CloneImage(image,0,0,MagickTrue,exception);
   if (separate_image == (Image *) NULL)
     return((Image *) NULL);
@@ -392,7 +397,7 @@ MagickExport MagickBooleanType SeparateImageChannel(Image *image,
     y;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
@@ -407,8 +412,8 @@ MagickExport MagickBooleanType SeparateImageChannel(Image *image,
   exception=(&image->exception);
   image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(image,image,image->rows,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(image,image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -572,7 +577,7 @@ MagickExport Image *SeparateImages(const Image *image,const ChannelType channel,
     *separate_image;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   images=NewImageList();
@@ -657,7 +662,7 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
   assert(image != (Image *) NULL);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   exception=(&image->exception);
   status=MagickTrue;
   switch (alpha_type)
@@ -677,8 +682,8 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
         break;
       image_view=AcquireAuthenticCacheView(image,exception);
       #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp parallel for schedule(static,4) shared(status) \
-          magick_threads(image,image,image->rows,1)
+        #pragma omp parallel for schedule(static) shared(status) \
+          magick_number_threads(image,image,image->rows,1)
       #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
@@ -748,8 +753,8 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       exception=(&image->exception);
       image_view=AcquireAuthenticCacheView(image,exception);
       #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp parallel for schedule(static,4) shared(status) \
-          magick_threads(image,image,image->rows,1)
+        #pragma omp parallel for schedule(static) shared(status) \
+          magick_number_threads(image,image,image->rows,1)
       #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
@@ -830,8 +835,8 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       image->matte=MagickTrue;
       image_view=AcquireAuthenticCacheView(image,exception);
       #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp parallel for schedule(static,4) shared(status) \
-          magick_threads(image,image,image->rows,1)
+        #pragma omp parallel for schedule(static) shared(status) \
+          magick_number_threads(image,image,image->rows,1)
       #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
@@ -900,13 +905,13 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
         NULL,&background);
       if (image->colorspace == CMYKColorspace)
         ConvertRGBToCMYK(&background);
-      (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
+      (void) memset(&pixel,0,sizeof(pixel));
       index=0;
       SetPixelPacket(image,&background,&pixel,&index);
       image_view=AcquireAuthenticCacheView(image,exception);
       #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp parallel for schedule(static,4) shared(status) \
-          magick_threads(image,image,image->rows,1)
+        #pragma omp parallel for schedule(static) shared(status) \
+          magick_number_threads(image,image,image->rows,1)
       #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
