@@ -17,13 +17,13 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -81,6 +81,8 @@ static const CoderMapInfo
   CoderMap[] =
   {
     { "3FR", "DNG" },
+    { "3GP", "MPEG" },
+    { "3G2", "MPEG" },
     { "8BIM", "META" },
     { "8BIMTEXT", "META" },
     { "8BIMWTEXT", "META" },
@@ -132,11 +134,13 @@ static const CoderMapInfo
     { "GIF87", "GIF" },
     { "G", "RAW" },
     { "GRANITE", "MAGICK" },
+    { "GRAYA", "GRAY" },
     { "GROUP4", "TIFF" },
     { "GV", "DOT" },
     { "HTM", "HTML" },
     { "ICB", "TGA" },
     { "ICO", "ICON" },
+    { "ICODIB", "DIB" },
     { "IIQ", "DNG" },
     { "K25", "DNG" },
     { "KDC", "DNG" },
@@ -204,7 +208,6 @@ static const CoderMapInfo
     { "PFB", "TTF" },
     { "PFM", "PNM" },
     { "PGM", "PNM" },
-    { "PGX", "JP2" },
     { "PICON", "XPM" },
     { "PJPEG", "JPEG" },
     { "PM", "XPM" },
@@ -372,12 +375,12 @@ static SplayTreeInfo *AcquireCoderCache(const char *filename,
           ResourceLimitError,"MemoryAllocationFailed","`%s'",p->name);
         continue;
       }
-    (void) ResetMagickMemory(coder_info,0,sizeof(*coder_info));
+    (void) memset(coder_info,0,sizeof(*coder_info));
     coder_info->path=(char *) "[built-in]";
     coder_info->magick=(char *) p->magick;
     coder_info->name=(char *) p->name;
     coder_info->exempt=MagickTrue;
-    coder_info->signature=MagickSignature;
+    coder_info->signature=MagickCoreSignature;
     status&=AddValueToSplayTree(cache,ConstantString(coder_info->magick),
       coder_info);
     if (status == MagickFalse)
@@ -469,23 +472,12 @@ MagickExport void CoderComponentTerminus(void)
 MagickExport const CoderInfo *GetCoderInfo(const char *name,
   ExceptionInfo *exception)
 {
-  const CoderInfo
-    *coder_info;
-
   assert(exception != (ExceptionInfo *) NULL);
   if (IsCoderTreeInstantiated(exception) == MagickFalse)
     return((const CoderInfo *) NULL);
-  LockSemaphoreInfo(coder_semaphore);
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
-    {
-      ResetSplayTreeIterator(coder_cache);
-      coder_info=(const CoderInfo *) GetNextValueInSplayTree(coder_cache);
-      UnlockSemaphoreInfo(coder_semaphore);
-      return(coder_info);
-    }
-  coder_info=(const CoderInfo *) GetValueFromSplayTree(coder_cache,name);
-  UnlockSemaphoreInfo(coder_semaphore);
-  return(coder_info);
+    return((const CoderInfo *) GetRootValueFromSplayTree(coder_cache));
+  return((const CoderInfo *) GetValueFromSplayTree(coder_cache,name));
 }
 
 /*
@@ -872,7 +864,7 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
           GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
-              if (depth > 200)
+              if (depth > MagickMaxRecursionDepth)
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   ConfigureError,"IncludeNodeNestedTooDeeply","`%s'",token);
               else
@@ -909,10 +901,10 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
         coder_info=(CoderInfo *) AcquireMagickMemory(sizeof(*coder_info));
         if (coder_info == (CoderInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-        (void) ResetMagickMemory(coder_info,0,sizeof(*coder_info));
+        (void) memset(coder_info,0,sizeof(*coder_info));
         coder_info->path=ConstantString(filename);
         coder_info->exempt=MagickFalse;
-        coder_info->signature=MagickSignature;
+        coder_info->signature=MagickCoreSignature;
         continue;
       }
     if (coder_info == (CoderInfo *) NULL)
