@@ -17,13 +17,13 @@
 %                              January 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -47,6 +47,7 @@
 #include "magick/geometry.h"
 #include "magick/image-private.h"
 #include "magick/memory_.h"
+#include "magick/pixel-accessor.h"
 #include "magick/string_.h"
 #include "magick/string-private.h"
 #include "magick/token.h"
@@ -179,7 +180,6 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
         break;
       }
       case '-':
-      case '.':
       case ',':
       case '+':
       case '0':
@@ -199,6 +199,18 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
         p++;
         break;
       }
+      case '.':
+      {
+        p++;
+        flags|=DecimalValue;
+        break;
+      }
+      case ':':
+      {
+        p++;
+        flags|=AspectRatioValue;
+        break;
+      }
       default:
         return(flags);
     }
@@ -228,7 +240,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
               if (LocaleNCompare(p,"0x",2) == 0)
                 *width=(size_t) strtol(p,&p,10);
               else
-                *width=(size_t) floor(StringToDouble(p,&p)+0.5);
+                *width=((size_t) floor(StringToDouble(p,&p)+0.5)) & 0x7fffffff;
             }
           if (p != q)
             flags|=WidthValue;
@@ -247,7 +259,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
               */
               q=p;
               if (height != (size_t *) NULL)
-                *height=(size_t) floor(StringToDouble(p,&p)+0.5);
+                *height=((size_t) floor(StringToDouble(p,&p)+0.5)) & 0x7fffffff;
               if (p != q)
                 flags|=HeightValue;
             }
@@ -266,7 +278,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
       }
       q=p;
       if (x != (ssize_t *) NULL)
-        *x=(ssize_t) ceil(StringToDouble(p,&p)-0.5);
+        *x=((ssize_t) ceil(StringToDouble(p,&p)-0.5)) & 0x7fffffff;
       if (p != q)
         {
           flags|=XValue;
@@ -287,7 +299,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
       }
       q=p;
       if (y != (ssize_t *) NULL)
-        *y=(ssize_t) ceil(StringToDouble(p,&p)-0.5);
+        *y=((ssize_t) ceil(StringToDouble(p,&p)-0.5)) & 0x7fffffff;
       if (p != q)
         {
           flags|=YValue;
@@ -305,7 +317,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
         }
       if (((flags & SeparatorValue) != 0) && ((flags & WidthValue) == 0) &&
           (height != (size_t *) NULL) && (width != (size_t *) NULL))
-            *width=(*height);
+        *width=(*height);
     }
 #if 0
   /*
@@ -412,6 +424,9 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
       MagickPageSize("c5", "459x649"),
       MagickPageSize("c6", "323x459"),
       MagickPageSize("c7", "230x323"),
+      MagickPageSize("csheet", "1224x1584"),
+      MagickPageSize("dsheet", "1584x2448"),
+      MagickPageSize("esheet", "2448x3168"),
       MagickPageSize("executive", "540x720"),
       MagickPageSize("flsa", "612x936"),
       MagickPageSize("flse", "612x936"),
@@ -439,6 +454,7 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
       MagickPageSize("legal", "612x1008"),
       MagickPageSize("letter", "612x792"),
       MagickPageSize("lettersmall", "612x792"),
+      MagickPageSize("monarch", "279x540"),
       MagickPageSize("quarto", "610x780"),
       MagickPageSize("statement", "396x612"),
       MagickPageSize("tabloid", "792x1224")
@@ -452,7 +468,7 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
 
   assert(page_geometry != (char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",page_geometry);
-  CopyMagickString(page,page_geometry,MaxTextExtent);
+  (void) CopyMagickString(page,page_geometry,MaxTextExtent);
   for (i=0; i < (ssize_t) (sizeof(PageSizes)/sizeof(PageSizes[0])); i++)
   {
     int
@@ -858,6 +874,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
     Remove whitespaces meta characters from geometry specification.
   */
   assert(geometry_info != (GeometryInfo *) NULL);
+  (void) memset(geometry_info,0,sizeof(*geometry_info));
   flags=NoValue;
   if ((geometry == (char *) NULL) || (*geometry == '\0'))
     return(flags);
@@ -937,7 +954,6 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       case '8':
       case '9':
       case '/':
-      case ':':
       case 215:
       case 'e':
       case 'E':
@@ -951,6 +967,12 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         flags|=DecimalValue;
         break;
       }
+      case ':':
+      {
+        p++;
+        flags|=AspectRatioValue;
+        break;
+      }
       default:
         return(NoValue);
     }
@@ -1089,9 +1111,10 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       /*
         Normalize sampling factor (e.g. 4:2:2 => 2x1).
       */
-      geometry_info->rho/=geometry_info->sigma;
+      if ((flags & SigmaValue) != 0)
+        geometry_info->rho*=PerceptibleReciprocal(geometry_info->sigma);
       geometry_info->sigma=1.0;
-      if (geometry_info->xi == 0.0)
+      if (((flags & XiValue) != 0) && (geometry_info->xi == 0.0))
         geometry_info->sigma=2.0;
     }
   if (((flags & SigmaValue) == 0) && ((flags & XiValue) != 0) &&
@@ -1212,6 +1235,36 @@ MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
       region_info->width=(size_t) floor((scale.x*image->columns/100.0)+0.5);
       region_info->height=(size_t) floor((scale.y*image->rows/100.0)+0.5);
     }
+  if ((flags & AspectRatioValue) != 0)
+    {
+      double
+        geometry_ratio,
+        image_ratio;
+
+      GeometryInfo
+        geometry_info;
+
+      /*
+        Geometry is a relative to image size and aspect ratio.
+      */
+      if (image->gravity != UndefinedGravity)
+        flags|=XValue | YValue;
+      (void) ParseGeometry(geometry,&geometry_info);
+      geometry_ratio=geometry_info.rho;
+      image_ratio=(double) image->columns/image->rows;
+      if (geometry_ratio >= image_ratio)
+        {
+          region_info->width=image->columns;
+          region_info->height=(size_t) floor((double) (image->rows*image_ratio/
+            geometry_ratio)+0.5);
+        }
+      else
+        {
+          region_info->width=(size_t) floor((double) (image->columns*
+            geometry_ratio/image_ratio)+0.5);
+          region_info->height=image->rows;
+        }
+    }
   /*
     Adjust offset according to gravity setting.
   */
@@ -1318,8 +1371,38 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       scale.y=geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
         scale.y=scale.x;
-      *width=(size_t) floor(scale.x*former_width/100.0+0.5);
-      *height=(size_t) floor(scale.y*former_height/100.0+0.5);
+      *width=(size_t) MagickMax(floor(scale.x*former_width/100.0+0.5),1.0);
+      *height=(size_t) MagickMax(floor(scale.y*former_height/100.0+0.5),1.0);
+      former_width=(*width);
+      former_height=(*height);
+    }
+  if ((flags & AspectRatioValue) != 0)
+    {
+      double
+        geometry_ratio,
+        image_ratio;
+
+      GeometryInfo
+        geometry_info;
+
+      /*
+        Geometry is a relative to image size and aspect ratio.
+      */
+      (void) ParseGeometry(geometry,&geometry_info);
+      geometry_ratio=geometry_info.rho;
+      image_ratio=(double) former_width*PerceptibleReciprocal(former_height);
+      if (geometry_ratio >= image_ratio)
+        {
+          *width=former_width;
+          *height=(size_t) floor((double) (former_height*image_ratio/
+            geometry_ratio)+0.5);
+        }
+      else
+        {
+          *width=(size_t) floor((double) (former_width*geometry_ratio/
+            image_ratio)+0.5);
+          *height=former_height;
+        }
       former_width=(*width);
       former_height=(*height);
     }
@@ -1401,12 +1484,14 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       (void) ParseGeometry(geometry,&geometry_info);
       area=geometry_info.rho+sqrt(MagickEpsilon);
       distance=sqrt((double) former_width*former_height);
-      scale.x=(double) former_width/(distance/sqrt(area));
-      scale.y=(double) former_height/(distance/sqrt(area));
+      scale.x=(double) former_width*PerceptibleReciprocal(distance/sqrt(area));
+      scale.y=(double) former_height*PerceptibleReciprocal(distance/sqrt(area));
       if ((scale.x < (double) *width) || (scale.y < (double) *height))
         {
-          *width=(unsigned long) (former_width/(distance/sqrt(area)));
-          *height=(unsigned long) (former_height/(distance/sqrt(area)));
+          *width=(unsigned long) (former_width*PerceptibleReciprocal(
+            distance/sqrt(area)));
+          *height=(unsigned long) (former_height*PerceptibleReciprocal(
+            distance/sqrt(area)));
         }
       former_width=(*width);
       former_height=(*height);
@@ -1558,11 +1643,11 @@ MagickExport MagickStatusType ParseRegionGeometry(const Image *image,
 MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (RectangleInfo *) NULL);
-  (void) ResetMagickMemory(geometry,0,sizeof(*geometry));
+  (void) memset(geometry,0,sizeof(*geometry));
   geometry->width=image->columns;
   geometry->height=image->rows;
 }
@@ -1593,5 +1678,5 @@ MagickExport void SetGeometryInfo(GeometryInfo *geometry_info)
 {
   assert(geometry_info != (GeometryInfo *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  (void) ResetMagickMemory(geometry_info,0,sizeof(*geometry_info));
+  (void) memset(geometry_info,0,sizeof(*geometry_info));
 }
