@@ -17,13 +17,13 @@
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -124,6 +124,7 @@ static const MagicMapInfo
     { "HDF", 1, MagicPattern("HDF") },
     { "HDR", 0, MagicPattern("#?RADIANCE") },
     { "HDR", 0, MagicPattern("#?RGBE") },
+    { "HEIC", 8, MagicPattern("heic") },
     { "HPGL", 0, MagicPattern("IN;") },
     { "HTML", 1, MagicPattern("HTML") },
     { "HTML", 1, MagicPattern("html") },
@@ -153,7 +154,8 @@ static const MagicMapInfo
     { "PES", 0, MagicPattern("#PES") },
     { "PFA", 0, MagicPattern("%!PS-AdobeFont-1.0") },
     { "PFB", 6, MagicPattern("%!PS-AdobeFont-1.0") },
-    { "PGX", 0, MagicPattern("\050\107\020\115\046") },
+    { "PGX", 0, MagicPattern("PG ML") },
+    { "PGX", 0, MagicPattern("PG LM") },
     { "PICT", 522, MagicPattern("\000\021\002\377\014\000") },
     { "PNG", 0, MagicPattern("\211PNG\r\n\032\n") },
     { "PBM", 0, MagicPattern("P1") },
@@ -179,6 +181,8 @@ static const MagicMapInfo
     { "SUN", 0, MagicPattern("\131\246\152\225") },
     { "SVG", 1, MagicPattern("?XML") },
     { "SVG", 1, MagicPattern("?xml") },
+    { "SVG", 1, MagicPattern("SVG") },
+    { "SVG", 1, MagicPattern("svg") },
     { "TIFF", 0, MagicPattern("\115\115\000\052") },
     { "TIFF", 0, MagicPattern("\111\111\052\000") },
     { "TIFF64", 0, MagicPattern("\115\115\000\053\000\010\000\000") },
@@ -246,10 +250,8 @@ static int CompareMagickInfoSize(const void *a,const void *b)
 
   ma=(MagicInfo *) a;
   mb=(MagicInfo *) b;
-
   if (ma->offset != mb->offset)
     return((int) (ma->offset-mb->offset));
-
   return((int) (mb->length-ma->length));
 }
 
@@ -315,7 +317,7 @@ static LinkedListInfo *AcquireMagicCache(const char *filename,
           ResourceLimitError,"MemoryAllocationFailed","`%s'",p->name);
         continue;
       }
-    (void) ResetMagickMemory(magic_info,0,sizeof(*magic_info));
+    (void) memset(magic_info,0,sizeof(*magic_info));
     magic_info->path=(char *) "[built-in]";
     magic_info->name=(char *) p->name;
     magic_info->offset=p->offset;
@@ -323,7 +325,7 @@ static LinkedListInfo *AcquireMagicCache(const char *filename,
     magic_info->magic=(unsigned char *) p->magic;
     magic_info->length=p->length;
     magic_info->exempt=MagickTrue;
-    magic_info->signature=MagickSignature;
+    magic_info->signature=MagickCoreSignature;
     status&=InsertValueInSortedLinkedList(cache,CompareMagickInfoSize,
       NULL,magic_info);
     if (status == MagickFalse)
@@ -611,7 +613,7 @@ MagickExport const char *GetMagicName(const MagicInfo *magic_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magic_info != (MagicInfo *) NULL);
-  assert(magic_info->signature == MagickSignature);
+  assert(magic_info->signature == MagickCoreSignature);
   return(magic_info->name);
 }
 
@@ -840,7 +842,7 @@ static MagickBooleanType LoadMagicCache(LinkedListInfo *cache,const char *xml,
           GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
-              if (depth > 200)
+              if (depth > MagickMaxRecursionDepth)
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
               else
@@ -877,10 +879,10 @@ static MagickBooleanType LoadMagicCache(LinkedListInfo *cache,const char *xml,
         magic_info=(MagicInfo *) AcquireMagickMemory(sizeof(*magic_info));
         if (magic_info == (MagicInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-        (void) ResetMagickMemory(magic_info,0,sizeof(*magic_info));
+        (void) memset(magic_info,0,sizeof(*magic_info));
         magic_info->path=ConstantString(filename);
         magic_info->exempt=MagickFalse;
-        magic_info->signature=MagickSignature;
+        magic_info->signature=MagickCoreSignature;
         continue;
       }
     if (magic_info == (MagicInfo *) NULL)
