@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
 
-    http://www.imagemagick.org/script/license.php
+    https://www.imagemagick.org/script/license.php
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -27,37 +27,45 @@ extern "C" {
 #if !defined(MAGICKCORE_QUANTUM_DEPTH)
 #define MAGICKCORE_QUANTUM_DEPTH  16
 #endif
+#if !defined(MagickPathExtent)
+#define MagickPathExtent  MaxTextExtent
+#endif
 
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__MINGW32__) && !defined(__MINGW64__)
-#  define MagickLLConstant(c)  (MagickOffsetType) (c ## i64)
-#  define MagickULLConstant(c)  (MagickSizeType) (c ## ui64)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__MINGW32__)
+#  define MagickLLConstant(c)  ((MagickOffsetType) (c ## i64))
+#  define MagickULLConstant(c)  ((MagickSizeType) (c ## ui64))
 #else
-#  define MagickLLConstant(c)  (MagickOffsetType) (c ## LL)
-#  define MagickULLConstant(c)  (MagickSizeType) (c ## ULL)
+#  define MagickLLConstant(c)  ((MagickOffsetType) (c ## LL))
+#  define MagickULLConstant(c)  ((MagickSizeType) (c ## ULL))
+#endif
+
+#if MAGICKCORE_SIZEOF_FLOAT_T == 0
+typedef float MagickFloatType;
+#elif (MAGICKCORE_SIZEOF_FLOAT_T == MAGICKCORE_SIZEOF_FLOAT)
+typedef float MagickFloatType;
+#elif (MAGICKCORE_SIZEOF_FLOAT_T == MAGICKCORE_SIZEOF_DOUBLE)
+typedef double MagickFloatType;
+#elif (MAGICKCORE_SIZEOF_FLOAT_T == MAGICKCORE_SIZEOF_LONG_DOUBLE)
+typedef long double MagickFloatType;
+#else
+#error Your MagickFloatType type is neither a float, nor a double, nor a long double
+#endif
+#if MAGICKCORE_SIZEOF_DOUBLE_T == 0
+typedef double MagickDoubleType;
+#elif (MAGICKCORE_SIZEOF_DOUBLE_T == MAGICKCORE_SIZEOF_DOUBLE)
+typedef double MagickDoubleType;
+#elif (MAGICKCORE_SIZEOF_DOUBLE_T == MAGICKCORE_SIZEOF_LONG_DOUBLE)
+typedef long double MagickDoubleType;
+#else
+#error Your MagickDoubleType type is neither a float, nor a double, nor a long double
 #endif
 
 #if (MAGICKCORE_QUANTUM_DEPTH == 8)
 #define MaxColormapSize  256UL
 #define MaxMap  255UL
-
-/*
-  Float_t is not an ABI type.
-*/
-#if MAGICKCORE_SIZEOF_FLOAT_T == 0
-typedef float MagickRealType;
-#elif (MAGICKCORE_SIZEOF_FLOAT_T == MAGICKCORE_SIZEOF_FLOAT)
-typedef float MagickRealType;
-#elif (MAGICKCORE_SIZEOF_FLOAT_T == MAGICKCORE_SIZEOF_DOUBLE)
-typedef double MagickRealType;
-#elif (MAGICKCORE_SIZEOF_FLOAT_T == MAGICKCORE_SIZEOF_LONG_DOUBLE)
-typedef long double MagickRealType;
-#else
-# error Your float_t type is neither a float, nor a double, nor a long double
-#endif
-
 typedef ssize_t SignedQuantum;
 #if defined(MAGICKCORE_HDRI_SUPPORT)
-typedef float Quantum;
+typedef MagickFloatType Quantum;
 #define QuantumRange  255.0
 #define QuantumFormat  "%g"
 #else
@@ -68,25 +76,9 @@ typedef unsigned char Quantum;
 #elif (MAGICKCORE_QUANTUM_DEPTH == 16)
 #define MaxColormapSize  65536UL
 #define MaxMap  65535UL
-
-/*
-  Float_t is not an ABI type.
-*/
-#if MAGICKCORE_SIZEOF_FLOAT_T == 0
-typedef float MagickRealType;
-#elif (MAGICKCORE_SIZEOF_FLOAT_T == MAGICKCORE_SIZEOF_FLOAT)
-typedef float MagickRealType;
-#elif (MAGICKCORE_SIZEOF_FLOAT_T == MAGICKCORE_SIZEOF_DOUBLE)
-typedef double MagickRealType;
-#elif (MAGICKCORE_SIZEOF_FLOAT_T == MAGICKCORE_SIZEOF_LONG_DOUBLE)
-typedef long double MagickRealType;
-#else
-# error Your float_t type is neither a float, nor a double, nor a long double
-#endif
-
 typedef ssize_t SignedQuantum;
 #if defined(MAGICKCORE_HDRI_SUPPORT)
-typedef float Quantum;
+typedef MagickFloatType Quantum;
 #define QuantumRange  65535.0
 #define QuantumFormat  "%g"
 #else
@@ -97,23 +89,9 @@ typedef unsigned short Quantum;
 #elif (MAGICKCORE_QUANTUM_DEPTH == 32)
 #define MaxColormapSize  65536UL
 #define MaxMap  65535UL
-
-/*
-  Double_t is not an ABI type.
-*/
-#if MAGICKCORE_SIZEOF_DOUBLE_T == 0
-typedef double MagickRealType;
-#elif (MAGICKCORE_SIZEOF_DOUBLE_T == MAGICKCORE_SIZEOF_DOUBLE)
-typedef double MagickRealType;
-#elif (MAGICKCORE_SIZEOF_DOUBLE_T == MAGICKCORE_SIZEOF_LONG_DOUBLE)
-typedef long double MagickRealType;
-#else
-# error Your double_t type is neither a float, nor a double, nor a long double
-#endif
-
-typedef double SignedQuantum;
+typedef MagickDoubleType SignedQuantum;
 #if defined(MAGICKCORE_HDRI_SUPPORT)
-typedef double Quantum;
+typedef MagickDoubleType Quantum;
 #define QuantumRange  4294967295.0
 #define QuantumFormat  "%g"
 #else
@@ -125,10 +103,8 @@ typedef unsigned int Quantum;
 #define MAGICKCORE_HDRI_SUPPORT 1
 #define MaxColormapSize  65536UL
 #define MaxMap  65535UL
-
-typedef long double MagickRealType;
-typedef long double SignedQuantum;
-typedef long double Quantum;
+typedef MagickDoubleType SignedQuantum;
+typedef MagickDoubleType Quantum;
 #define QuantumRange  18446744073709551615.0
 #define QuantumFormat  "%g"
 #else
@@ -136,7 +112,7 @@ typedef long double Quantum;
 # error "MAGICKCORE_QUANTUM_DEPTH must be one of 8, 16, 32, or 64"
 #endif
 #endif
-#define MagickEpsilon  (1.0e-15)
+#define MagickEpsilon  (1.0e-12)
 #define MagickMaximumValue  1.79769313486231570E+308
 #define MagickMinimumValue   2.22507385850720140E-308
 #define QuantumScale  ((double) 1.0/(double) QuantumRange)
@@ -144,6 +120,8 @@ typedef long double Quantum;
 /*
   Typedef declarations.
 */
+typedef MagickDoubleType MagickRealType;
+
 typedef unsigned int MagickStatusType;
 #if !defined(MAGICKCORE_WINDOWS_SUPPORT)
 #if (MAGICKCORE_SIZEOF_UNSIGNED_LONG_LONG == 8)
@@ -214,6 +192,29 @@ typedef enum
   MagickTrue = 1
 } MagickBooleanType;
 
+/*
+  The IsNaN test is for special floating point numbers of value Nan (not a
+  number). NaN's are defined as part of the IEEE standard for floating point
+  number representation, and need to be watched out for. Morphology Kernels
+  often use these special numbers as neighbourhood masks.
+
+  The special property that two NaN's are never equal, even if they are from
+  the same variable allows you to test if a value is special NaN value.
+
+  The macros are thus is only true if the value given is NaN.
+*/
+#if defined(MAGICKCORE_HAVE_ISNAN)
+#  define IsNaN(a) isnan(a)
+#elif defined(_MSC_VER) && (_MSC_VER >= 1310)
+#  include <float.h>
+#  define IsNaN(a) _isnan(a)
+#else
+#  define IsNaN(a) ((a) != (a))
+#endif
+#if !defined(INFINITY)
+#  define INFINITY (-logf(0f))
+#endif
+
 typedef struct _BlobInfo BlobInfo;
 
 typedef struct _ExceptionInfo ExceptionInfo;
