@@ -16,13 +16,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -50,6 +50,8 @@
 #include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/policy.h"
+#include "magick/policy-private.h"
+#include "magick/resource_.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -152,7 +154,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o filename: the font file name.
+%    o filename: the policy configuration file name.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -176,7 +178,10 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
   if (cache == (LinkedListInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
-#if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+#if defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+  status=LoadPolicyCache(cache,ZeroConfigurationPolicy,"[zero-configuration]",0,
+    exception);
+#else
   {
     const StringInfo
       *option;
@@ -188,8 +193,8 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
     option=(const StringInfo *) GetNextValueInLinkedList(options);
     while (option != (const StringInfo *) NULL)
     {
-      status&=LoadPolicyCache(cache,(const char *)
-        GetStringInfoDatum(option),GetStringInfoPath(option),0,exception);
+      status&=LoadPolicyCache(cache,(const char *) GetStringInfoDatum(option),
+        GetStringInfoPath(option),0,exception);
       option=(const StringInfo *) GetNextValueInLinkedList(options);
     }
     options=DestroyConfigureOptions(options);
@@ -214,7 +219,7 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
           ResourceLimitError,"MemoryAllocationFailed","`%s'",p->name);
         continue;
       }
-    (void) ResetMagickMemory(policy_info,0,sizeof(*policy_info));
+    (void) memset(policy_info,0,sizeof(*policy_info));
     policy_info->path=(char *) "[built-in]";
     policy_info->domain=p->domain;
     policy_info->rights=p->rights;
@@ -222,7 +227,7 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
     policy_info->pattern=(char *) p->pattern;
     policy_info->value=(char *) p->value;
     policy_info->exempt=MagickTrue;
-    policy_info->signature=MagickSignature;
+    policy_info->signature=MagickCoreSignature;
     status&=AppendValueToLinkedList(cache,policy_info);
     if (status == MagickFalse)
       (void) ThrowMagickException(exception,GetMagickModule(),
@@ -259,7 +264,10 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
 static PolicyInfo *GetPolicyInfo(const char *name,ExceptionInfo *exception)
 {
   char
-    policyname[MaxTextExtent];
+    policyname[MagickPathExtent];
+
+  PolicyDomain
+    domain;
 
   register PolicyInfo
     *p;
@@ -275,14 +283,28 @@ static PolicyInfo *GetPolicyInfo(const char *name,ExceptionInfo *exception)
   */
   *policyname='\0';
   if (name != (const char *) NULL)
-    (void) CopyMagickString(policyname,name,MaxTextExtent);
+    (void) CopyMagickString(policyname,name,MagickPathExtent);
   for (q=policyname; *q != '\0'; q++)
   {
     if (isspace((int) ((unsigned char) *q)) == 0)
       continue;
-    (void) CopyMagickString(q,q+1,MaxTextExtent);
+    (void) CopyMagickString(q,q+1,MagickPathExtent);
     q--;
   }
+  /*
+    Strip domain from policy name (e.g. resource:map).
+  */
+  domain=UndefinedPolicyDomain;
+  for (q=policyname; *q != '\0'; q++)
+  {
+    if (*q != ':')
+      continue;
+    *q='\0';
+    domain=(PolicyDomain) ParseCommandOption(MagickPolicyDomainOptions,
+      MagickTrue,policyname);
+    (void) CopyMagickString(policyname,q+1,MagickPathExtent);
+    break;
+  }
   /*
     Search for policy tag.
   */
@@ -296,8 +318,9 @@ static PolicyInfo *GetPolicyInfo(const char *name,ExceptionInfo *exception)
     }
   while (p != (PolicyInfo *) NULL)
   {
-    if (LocaleCompare(policyname,p->name) == 0)
-      break;
+    if ((domain == UndefinedPolicyDomain) || (p->domain == domain))
+      if (LocaleCompare(policyname,p->name) == 0)
+        break;
     p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
   }
   if (p != (PolicyInfo *) NULL)
@@ -590,20 +613,20 @@ MagickExport MagickBooleanType IsRightsAuthorized(const PolicyDomain domain,
   LockSemaphoreInfo(policy_semaphore);
   ResetLinkedListIterator(policy_cache);
   p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
-  while ((p != (PolicyInfo *) NULL) && (authorized != MagickFalse))
+  while (p != (PolicyInfo *) NULL)
   {
     if ((p->domain == domain) &&
         (GlobExpression(pattern,p->pattern,MagickFalse) != MagickFalse))
       {
-        if (((rights & ReadPolicyRights) != 0) &&
-            ((p->rights & ReadPolicyRights) == 0))
-          authorized=MagickFalse;
-        if (((rights & WritePolicyRights) != 0) &&
-            ((p->rights & WritePolicyRights) == 0))
-          authorized=MagickFalse;
-        if (((rights & ExecutePolicyRights) != 0) &&
-            ((p->rights & ExecutePolicyRights) == 0))
-          authorized=MagickFalse;
+        if ((rights & ReadPolicyRights) != 0)
+          authorized=(p->rights & ReadPolicyRights) != 0 ? MagickTrue :
+            MagickFalse;
+        if ((rights & WritePolicyRights) != 0)
+          authorized=(p->rights & WritePolicyRights) != 0 ? MagickTrue :
+            MagickFalse;
+        if ((rights & ExecutePolicyRights) != 0)
+          authorized=(p->rights & ExecutePolicyRights) != 0 ? MagickTrue :
+            MagickFalse;
       }
     p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
   }
@@ -805,7 +828,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
           GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
-              if (depth > 200)
+              if (depth > MagickMaxRecursionDepth)
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
               else
@@ -842,10 +865,10 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
         policy_info=(PolicyInfo *) AcquireMagickMemory(sizeof(*policy_info));
         if (policy_info == (PolicyInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-        (void) ResetMagickMemory(policy_info,0,sizeof(*policy_info));
+        (void) memset(policy_info,0,sizeof(*policy_info));
         policy_info->path=ConstantString(filename);
         policy_info->exempt=MagickFalse;
-        policy_info->signature=MagickSignature;
+        policy_info->signature=MagickCoreSignature;
         continue;
       }
     if (policy_info == (PolicyInfo *) NULL)
@@ -1013,3 +1036,58 @@ MagickExport void PolicyComponentTerminus(void)
   UnlockSemaphoreInfo(policy_semaphore);
   DestroySemaphoreInfo(&policy_semaphore);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%  S e t M a g i c k S e c u r i t y P o l i c y                              %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetMagickSecurityPolicy() sets the ImageMagick security policy.  It returns
+%  MagickFalse if the policy is already set or if the policy does not parse.
+%
+%  The format of the SetMagickSecurityPolicy method is:
+%
+%      MagickBooleanType SetMagickSecurityPolicy(const char *policy,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o policy: the security policy in the XML format.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+MagickExport MagickBooleanType SetMagickSecurityPolicy(const char *policy,
+  ExceptionInfo *exception)
+{
+  PolicyInfo
+    *p;
+
+  MagickBooleanType
+    status;
+
+  assert(exception != (ExceptionInfo *) NULL);
+  if (policy == (const char *) NULL)
+    return(MagickFalse);
+  if (IsPolicyCacheInstantiated(exception) == MagickFalse)
+    return(MagickFalse);
+  LockSemaphoreInfo(policy_semaphore);
+  ResetLinkedListIterator(policy_cache);
+  p=(PolicyInfo *) GetNextValueInLinkedList(policy_cache);
+  if ((p != (PolicyInfo *) NULL) && (p->domain != UndefinedPolicyDomain))
+    {
+      UnlockSemaphoreInfo(policy_semaphore);
+      return(MagickFalse);
+    }
+  UnlockSemaphoreInfo(policy_semaphore);
+  status=LoadPolicyCache(policy_cache,policy,"[user-policy]",0,exception);
+  if (status == MagickFalse)
+    return(MagickFalse);
+  return(ResourceComponentGenesis());
+}
