@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -73,6 +73,33 @@
 #include "lcms2.h"
 #endif
 #endif
+
+/*
+  Definitions
+*/
+#define LCMSHDRI
+#if !defined(MAGICKCORE_HDRI_SUPPORT)
+  #if (MAGICKCORE_QUANTUM_DEPTH == 8)
+  #undef LCMSHDRI
+  #define LCMSScaleSource(pixel)  ScaleQuantumToShort(pixel)
+  #define LCMSScaleTarget(pixel)  ScaleShortToQuantum(pixel)
+  typedef unsigned short
+    LCMSType;
+  #elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+  #undef LCMSHDRI
+  #define LCMSScaleSource(pixel)  (pixel)
+  #define LCMSScaleTarget(pixel)  (pixel)
+  typedef unsigned short
+    LCMSType;
+  #endif
+#endif
+
+#if defined(LCMSHDRI)
+#define LCMSScaleSource(pixel)  (source_scale*QuantumScale*(pixel))
+#define LCMSScaleTarget(pixel) ClampToQuantum(target_scale*QuantumRange*(pixel))
+typedef double
+  LCMSType;
+#endif
 
 /*
   Forward declarations
@@ -113,11 +140,11 @@ MagickExport MagickBooleanType CloneImageProfiles(Image *image,
   const Image *clone_image)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(clone_image != (const Image *) NULL);
-  assert(clone_image->signature == MagickSignature);
+  assert(clone_image->signature == MagickCoreSignature);
   image->color_profile.length=clone_image->color_profile.length;
   image->color_profile.info=clone_image->color_profile.info;
   image->iptc_profile.length=clone_image->iptc_profile.length;
@@ -159,7 +186,7 @@ MagickExport MagickBooleanType CloneImageProfiles(Image *image,
 MagickExport MagickBooleanType DeleteImageProfile(Image *image,const char *name)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->profiles == (SplayTreeInfo *) NULL)
@@ -243,7 +270,7 @@ MagickExport const StringInfo *GetImageProfile(const Image *image,
     *profile;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->profiles == (SplayTreeInfo *) NULL)
@@ -278,7 +305,7 @@ MagickExport const StringInfo *GetImageProfile(const Image *image,
 MagickExport char *GetNextImageProfile(const Image *image)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->profiles == (SplayTreeInfo *) NULL)
@@ -329,42 +356,42 @@ MagickExport char *GetNextImageProfile(const Image *image)
 
 #if defined(MAGICKCORE_LCMS_DELEGATE)
 
-static unsigned short **DestroyPixelThreadSet(unsigned short **pixels)
+static LCMSType **DestroyPixelThreadSet(LCMSType **pixels)
 {
   register ssize_t
     i;
 
-  assert(pixels != (unsigned short **) NULL);
+  if (pixels == (LCMSType **) NULL)
+    return((LCMSType **) NULL);
   for (i=0; i < (ssize_t) GetMagickResourceLimit(ThreadResource); i++)
-    if (pixels[i] != (unsigned short *) NULL)
-      pixels[i]=(unsigned short *) RelinquishMagickMemory(pixels[i]);
-  pixels=(unsigned short **) RelinquishMagickMemory(pixels);
+    if (pixels[i] != (LCMSType *) NULL)
+      pixels[i]=(LCMSType *) RelinquishMagickMemory(pixels[i]);
+  pixels=(LCMSType **) RelinquishMagickMemory(pixels);
   return(pixels);
 }
 
-static unsigned short **AcquirePixelThreadSet(const size_t columns,
+static LCMSType **AcquirePixelThreadSet(const size_t columns,
   const size_t channels)
 {
+  LCMSType
+    **pixels;
+
   register ssize_t
     i;
 
-  unsigned short
-    **pixels;
-
   size_t
     number_threads;
 
   number_threads=(size_t) GetMagickResourceLimit(ThreadResource);
-  pixels=(unsigned short **) AcquireQuantumMemory(number_threads,
-    sizeof(*pixels));
-  if (pixels == (unsigned short **) NULL)
-    return((unsigned short **) NULL);
-  (void) ResetMagickMemory(pixels,0,number_threads*sizeof(*pixels));
+  pixels=(LCMSType **) AcquireQuantumMemory(number_threads,sizeof(*pixels));
+  if (pixels == (LCMSType **) NULL)
+    return((LCMSType **) NULL);
+  (void) memset(pixels,0,number_threads*sizeof(*pixels));
   for (i=0; i < (ssize_t) number_threads; i++)
   {
-    pixels[i]=(unsigned short *) AcquireQuantumMemory(columns,channels*
+    pixels[i]=(LCMSType *) AcquireQuantumMemory(columns,channels*
       sizeof(**pixels));
-    if (pixels[i] == (unsigned short *) NULL)
+    if (pixels[i] == (LCMSType *) NULL)
       return(DestroyPixelThreadSet(pixels));
   }
   return(pixels);
@@ -402,7 +429,7 @@ static cmsHTRANSFORM *AcquireTransformThreadSet(Image *image,
     sizeof(*transform));
   if (transform == (cmsHTRANSFORM *) NULL)
     return((cmsHTRANSFORM *) NULL);
-  (void) ResetMagickMemory(transform,0,number_threads*sizeof(*transform));
+  (void) memset(transform,0,number_threads*sizeof(*transform));
   for (i=0; i < (ssize_t) number_threads; i++)
   {
     transform[i]=cmsCreateTransformTHR((cmsContext) image,source_profile,
@@ -712,7 +739,7 @@ static MagickBooleanType SetsRGBImageProfile(Image *image)
     status;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (GetImageProfile(image,"icc") != (const StringInfo *) NULL)
     return(MagickFalse);
   profile=AcquireStringInfo(sizeof(sRGBProfile));
@@ -745,7 +772,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
   magick_unreferenced(clone);
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(name != (const char *) NULL);
@@ -860,6 +887,16 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             int
               intent;
 
+            LCMSType
+              **magick_restrict source_pixels,
+              **magick_restrict target_pixels;
+
+#if defined(LCMSHDRI)
+            LCMSType
+              source_scale,
+              target_scale;
+#endif
+
             MagickOffsetType
               progress;
 
@@ -870,10 +907,6 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             ssize_t
               y;
 
-            unsigned short
-              **magick_restrict source_pixels,
-              **magick_restrict target_pixels;
-
             exception=(&image->exception);
             target_profile=(cmsHPROFILE) NULL;
             if (icc_profile != (StringInfo *) NULL)
@@ -886,155 +919,162 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                   ThrowProfileException(ResourceLimitError,
                     "ColorspaceColorProfileMismatch",name);
               }
+#if defined(LCMSHDRI)
+            source_scale=1.0;
+#endif
+            source_channels=3;
             switch (cmsGetColorSpace(source_profile))
             {
               case cmsSigCmykData:
               {
                 source_colorspace=CMYKColorspace;
-                source_type=(cmsUInt32Number) TYPE_CMYK_16;
                 source_channels=4;
+#if defined(LCMSHDRI)
+                source_type=(cmsUInt32Number) TYPE_CMYK_DBL;
+                source_scale=100.0;
+#else
+                source_type=(cmsUInt32Number) TYPE_CMYK_16;
+#endif
                 break;
               }
               case cmsSigGrayData:
               {
                 source_colorspace=GRAYColorspace;
-                source_type=(cmsUInt32Number) TYPE_GRAY_16;
                 source_channels=1;
+#if defined(LCMSHDRI)
+                source_type=(cmsUInt32Number) TYPE_GRAY_DBL;
+#else
+                source_type=(cmsUInt32Number) TYPE_GRAY_16;
+#endif
                 break;
               }
               case cmsSigLabData:
               {
                 source_colorspace=LabColorspace;
+#if defined(LCMSHDRI)
+                source_type=(cmsUInt32Number) TYPE_Lab_DBL;
+                source_scale=100.0;
+#else
                 source_type=(cmsUInt32Number) TYPE_Lab_16;
-                source_channels=3;
+#endif
                 break;
               }
+#if !defined(LCMSHDRI)
               case cmsSigLuvData:
               {
                 source_colorspace=YUVColorspace;
                 source_type=(cmsUInt32Number) TYPE_YUV_16;
-                source_channels=3;
                 break;
               }
+#endif
               case cmsSigRgbData:
               {
+#if defined(LCMSHDRI)
                 source_colorspace=sRGBColorspace;
+                source_type=(cmsUInt32Number) TYPE_RGB_DBL;
+#else
                 source_type=(cmsUInt32Number) TYPE_RGB_16;
-                source_channels=3;
+#endif
                 break;
               }
               case cmsSigXYZData:
               {
                 source_colorspace=XYZColorspace;
+#if defined(LCMSHDRI)
+                source_type=(cmsUInt32Number) TYPE_XYZ_DBL;
+#else
                 source_type=(cmsUInt32Number) TYPE_XYZ_16;
-                source_channels=3;
+#endif
                 break;
               }
+#if !defined(LCMSHDRI)
               case cmsSigYCbCrData:
               {
-                source_colorspace=YCbCrColorspace;
+                source_colorspace=YUVColorspace;
                 source_type=(cmsUInt32Number) TYPE_YCbCr_16;
-                source_channels=3;
                 break;
               }
+#endif
               default:
-              {
-                source_colorspace=UndefinedColorspace;
-                source_type=(cmsUInt32Number) TYPE_RGB_16;
-                source_channels=3;
-                break;
-              }
+                ThrowProfileException(ImageError,
+                  "ColorspaceColorProfileMismatch",name);
             }
+            (void) source_colorspace;
             signature=cmsGetPCS(source_profile);
             if (target_profile != (cmsHPROFILE) NULL)
               signature=cmsGetColorSpace(target_profile);
+#if defined(LCMSHDRI)
+            target_scale=1.0;
+#endif
+            target_channels=3;
             switch (signature)
             {
               case cmsSigCmykData:
               {
                 target_colorspace=CMYKColorspace;
-                target_type=(cmsUInt32Number) TYPE_CMYK_16;
                 target_channels=4;
-                break;
-              }
-              case cmsSigLabData:
-              {
-                target_colorspace=LabColorspace;
-                target_type=(cmsUInt32Number) TYPE_Lab_16;
-                target_channels=3;
+#if defined(LCMSHDRI)
+                target_type=(cmsUInt32Number) TYPE_CMYK_DBL;
+                target_scale=0.01;
+#else
+                target_type=(cmsUInt32Number) TYPE_CMYK_16;
+#endif
                 break;
               }
               case cmsSigGrayData:
               {
                 target_colorspace=GRAYColorspace;
-                target_type=(cmsUInt32Number) TYPE_GRAY_16;
                 target_channels=1;
+#if defined(LCMSHDRI)
+                target_type=(cmsUInt32Number) TYPE_GRAY_DBL;
+#else
+                target_type=(cmsUInt32Number) TYPE_GRAY_16;
+#endif
                 break;
               }
+              case cmsSigLabData:
+              {
+                target_colorspace=LabColorspace;
+#if defined(LCMSHDRI)
+                target_type=(cmsUInt32Number) TYPE_Lab_DBL;
+                target_scale=0.01;
+#else
+                target_type=(cmsUInt32Number) TYPE_Lab_16;
+#endif
+                break;
+              }
+#if !defined(LCMSHDRI)
               case cmsSigLuvData:
               {
                 target_colorspace=YUVColorspace;
                 target_type=(cmsUInt32Number) TYPE_YUV_16;
-                target_channels=3;
                 break;
               }
+#endif
               case cmsSigRgbData:
               {
                 target_colorspace=sRGBColorspace;
+#if defined(LCMSHDRI)
+                target_type=(cmsUInt32Number) TYPE_RGB_DBL;
+#else
                 target_type=(cmsUInt32Number) TYPE_RGB_16;
-                target_channels=3;
+#endif
                 break;
               }
               case cmsSigXYZData:
               {
                 target_colorspace=XYZColorspace;
+#if defined(LCMSHDRI)
+                target_type=(cmsUInt32Number) TYPE_XYZ_DBL;
+#else
                 target_type=(cmsUInt32Number) TYPE_XYZ_16;
-                target_channels=3;
-                break;
-              }
-              case cmsSigYCbCrData:
-              {
-                target_colorspace=YCbCrColorspace;
-                target_type=(cmsUInt32Number) TYPE_YCbCr_16;
-                target_channels=3;
+#endif
                 break;
               }
               default:
-              {
-                target_colorspace=UndefinedColorspace;
-                target_type=(cmsUInt32Number) TYPE_RGB_16;
-                target_channels=3;
-                break;
-              }
+                ThrowProfileException(ImageError,
+                  "ColorspaceColorProfileMismatch",name);
             }
-            if ((source_colorspace == UndefinedColorspace) ||
-                (target_colorspace == UndefinedColorspace))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
-             if ((source_colorspace == GRAYColorspace) &&
-                 (SetImageGray(image,exception) == MagickFalse))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
-             if ((source_colorspace == CMYKColorspace) &&
-                 (image->colorspace != CMYKColorspace))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
-             if ((source_colorspace == XYZColorspace) &&
-                 (image->colorspace != XYZColorspace))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
-             if ((source_colorspace == YCbCrColorspace) &&
-                 (image->colorspace != YCbCrColorspace))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
-             if ((source_colorspace != CMYKColorspace) &&
-                 (source_colorspace != GRAYColorspace) &&
-                 (source_colorspace != LabColorspace) &&
-                 (source_colorspace != XYZColorspace) &&
-                 (source_colorspace != YCbCrColorspace) &&
-                 (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
             switch (image->rendering_intent)
             {
               case AbsoluteIntent: intent=INTENT_ABSOLUTE_COLORIMETRIC; break;
@@ -1058,9 +1098,11 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             */
             source_pixels=AcquirePixelThreadSet(image->columns,source_channels);
             target_pixels=AcquirePixelThreadSet(image->columns,target_channels);
-            if ((source_pixels == (unsigned short **) NULL) ||
-                (target_pixels == (unsigned short **) NULL))
+            if ((source_pixels == (LCMSType **) NULL) ||
+                (target_pixels == (LCMSType **) NULL))
               {
+                target_pixels=DestroyPixelThreadSet(target_pixels);
+                source_pixels=DestroyPixelThreadSet(source_pixels);
                 transform=DestroyTransformThreadSet(transform);
                 ThrowProfileException(ResourceLimitError,
                   "MemoryAllocationFailed",image->filename);
@@ -1081,8 +1123,8 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             progress=0;
             image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp parallel for schedule(static,4) shared(status) \
-              magick_threads(image,image,image->rows,1)
+            #pragma omp parallel for schedule(static) shared(status) \
+              magick_number_threads(image,image,image->rows,1)
 #endif
             for (y=0; y < (ssize_t) image->rows; y++)
             {
@@ -1095,14 +1137,14 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               register IndexPacket
                 *magick_restrict indexes;
 
-              register ssize_t
-                x;
+              register LCMSType
+                *p;
 
               register PixelPacket
                 *magick_restrict q;
 
-              register unsigned short
-                *p;
+              register ssize_t
+                x;
 
               if (status == MagickFalse)
                 continue;
@@ -1117,14 +1159,14 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               p=source_pixels[id];
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                *p++=ScaleQuantumToShort(GetPixelRed(q));
+                *p++=LCMSScaleSource(GetPixelRed(q));
                 if (source_channels > 1)
                   {
-                    *p++=ScaleQuantumToShort(GetPixelGreen(q));
-                    *p++=ScaleQuantumToShort(GetPixelBlue(q));
+                    *p++=LCMSScaleSource(GetPixelGreen(q));
+                    *p++=LCMSScaleSource(GetPixelBlue(q));
                   }
                 if (source_channels > 3)
-                  *p++=ScaleQuantumToShort(GetPixelIndex(indexes+x));
+                  *p++=LCMSScaleSource(GetPixelIndex(indexes+x));
                 q++;
               }
               cmsDoTransform(transform[id],source_pixels[id],target_pixels[id],
@@ -1133,20 +1175,20 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               q-=image->columns;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                SetPixelRed(q,ScaleShortToQuantum(*p));
+                SetPixelRed(q,LCMSScaleTarget(*p));
                 SetPixelGreen(q,GetPixelRed(q));
                 SetPixelBlue(q,GetPixelRed(q));
                 p++;
                 if (target_channels > 1)
                   {
-                    SetPixelGreen(q,ScaleShortToQuantum(*p));
+                    SetPixelGreen(q,LCMSScaleTarget(*p));
                     p++;
-                    SetPixelBlue(q,ScaleShortToQuantum(*p));
+                    SetPixelBlue(q,LCMSScaleTarget(*p));
                     p++;
                   }
                 if (target_channels > 3)
                   {
-                    SetPixelIndex(indexes+x,ScaleShortToQuantum(*p));
+                    SetPixelIndex(indexes+x,LCMSScaleTarget(*p));
                     p++;
                   }
                 q++;
@@ -1196,7 +1238,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             target_pixels=DestroyPixelThreadSet(target_pixels);
             source_pixels=DestroyPixelThreadSet(source_pixels);
             transform=DestroyTransformThreadSet(transform);
-            if ((status != MagickFalse) ||
+            if ((status != MagickFalse) &&
                 (cmsGetDeviceClass(source_profile) != cmsSigLinkClass))
               status=SetImageProfile(image,name,profile);
             if (target_profile != (cmsHPROFILE) NULL)
@@ -1241,7 +1283,7 @@ MagickExport StringInfo *RemoveImageProfile(Image *image,const char *name)
     *profile;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->profiles == (SplayTreeInfo *) NULL)
@@ -1295,7 +1337,7 @@ MagickExport StringInfo *RemoveImageProfile(Image *image,const char *name)
 MagickExport void ResetImageProfileIterator(const Image *image)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->profiles == (SplayTreeInfo *) NULL)
@@ -1374,7 +1416,7 @@ static inline void WriteResourceLong(unsigned char *p,
   buffer[1]=(unsigned char) (quantum >> 16);
   buffer[2]=(unsigned char) (quantum >> 8);
   buffer[3]=(unsigned char) quantum;
-  (void) CopyMagickMemory(p,buffer,4);
+  (void) memcpy(p,buffer,4);
 }
 
 static void WriteTo8BimProfile(Image *image,const char *name,
@@ -1440,8 +1482,7 @@ static void WriteTo8BimProfile(Image *image,const char *name,
     count=(ssize_t) value;
     if ((count & 0x01) != 0)
       count++;
-    if ((count < 0) || (p > (datum+length-count)) ||
-        (count > (ssize_t) length))
+    if ((count < 0) || (p > (datum+length-count)) || (count > (ssize_t) length))
       break;
     if (id != profile_id)
       p+=count;
@@ -1463,7 +1504,7 @@ static void WriteTo8BimProfile(Image *image,const char *name,
           {
             offset=(q-datum);
             extract_profile=AcquireStringInfo(offset+extent);
-            (void) CopyMagickMemory(extract_profile->datum,datum,offset);
+            (void) memcpy(extract_profile->datum,datum,offset);
           }
         else
           {
@@ -1472,13 +1513,13 @@ static void WriteTo8BimProfile(Image *image,const char *name,
             if ((extract_extent & 0x01) != 0)
               extract_extent++;
             extract_profile=AcquireStringInfo(offset+extract_extent+extent);
-            (void) CopyMagickMemory(extract_profile->datum,datum,offset-4);
-            (void) WriteResourceLong(extract_profile->datum+offset-4,
-              (unsigned int) profile->length);
-            (void) CopyMagickMemory(extract_profile->datum+offset,
+            (void) memcpy(extract_profile->datum,datum,offset-4);
+            WriteResourceLong(extract_profile->datum+offset-4,(unsigned int)
+              profile->length);
+            (void) memcpy(extract_profile->datum+offset,
               profile->datum,profile->length);
           }
-        (void) CopyMagickMemory(extract_profile->datum+offset+extract_extent,
+        (void) memcpy(extract_profile->datum+offset+extract_extent,
           p+count,extent);
         (void) AddValueToSplayTree((SplayTreeInfo *) image->profiles,
           ConstantString("8bim"),CloneStringInfo(extract_profile));
@@ -1531,8 +1572,7 @@ static void GetProfilesFromResourceBlock(Image *image,
       break;
     p=ReadResourceLong(p,&value);
     count=(ssize_t) value;
-    if ((p > (datum+length-count)) || (count > (ssize_t) length) ||
-        (count < 0))
+    if ((p > (datum+length-count)) || (count > (ssize_t) length) || (count < 0))
       break;
     switch (id)
     {
@@ -1547,6 +1587,8 @@ static void GetProfilesFromResourceBlock(Image *image,
         /*
           Resolution.
         */
+        if (count < 10)
+          break;
         p=ReadResourceLong(p,&resolution);
         image->x_resolution=((double) resolution)/65536.0;
         p=ReadResourceShort(p,&units)+2;
@@ -1643,7 +1685,7 @@ static MagickBooleanType SetImageProfileInternal(Image *image,const char *name,
     status;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->profiles == (SplayTreeInfo *) NULL)
@@ -1767,7 +1809,7 @@ static inline signed short ReadProfileShort(const EndianType endian,
     }
   value=(unsigned short) buffer[0] << 8;
   value|=(unsigned short) buffer[1];
-  quantum.unsigned_value=(value & 0xffff);
+  quantum.unsigned_value=value & 0xffff;
   return(quantum.signed_value);
 }
 
@@ -1842,14 +1884,14 @@ static inline void WriteProfileLong(const EndianType endian,
       buffer[1]=(unsigned char) (value >> 8);
       buffer[2]=(unsigned char) (value >> 16);
       buffer[3]=(unsigned char) (value >> 24);
-      (void) CopyMagickMemory(p,buffer,4);
+      (void) memcpy(p,buffer,4);
       return;
     }
   buffer[0]=(unsigned char) (value >> 24);
   buffer[1]=(unsigned char) (value >> 16);
   buffer[2]=(unsigned char) (value >> 8);
   buffer[3]=(unsigned char) value;
-  (void) CopyMagickMemory(p,buffer,4);
+  (void) memcpy(p,buffer,4);
 }
 
 static void WriteProfileShort(const EndianType endian,
@@ -1862,12 +1904,12 @@ static void WriteProfileShort(const EndianType endian,
     {
       buffer[0]=(unsigned char) value;
       buffer[1]=(unsigned char) (value >> 8);
-      (void) CopyMagickMemory(p,buffer,2);
+      (void) memcpy(p,buffer,2);
       return;
     }
   buffer[0]=(unsigned char) (value >> 8);
   buffer[1]=(unsigned char) value;
-  (void) CopyMagickMemory(p,buffer,2);
+  (void) memcpy(p,buffer,2);
 }
 
 static MagickBooleanType Sync8BimProfile(Image *image,StringInfo *profile)
@@ -1900,9 +1942,10 @@ static MagickBooleanType Sync8BimProfile(Image *image,StringInfo *profile)
       return(MagickFalse);
     id=ReadProfileMSBShort(&p,&length);
     count=(ssize_t) ReadProfileByte(&p,&length);
-    if ((count > (ssize_t) length) || (count < 0))
+    if ((count >= (ssize_t) length) || (count < 0))
       return(MagickFalse);
     p+=count;
+    length-=count;
     if ((*p & 0x01) == 0)
       (void) ReadProfileByte(&p,&length);
     count=(ssize_t) ReadProfileMSBLong(&p,&length);
@@ -1911,17 +1954,17 @@ static MagickBooleanType Sync8BimProfile(Image *image,StringInfo *profile)
     if ((id == 0x3ED) && (count == 16))
       {
         if (image->units == PixelsPerCentimeterResolution)
-          WriteProfileLong(MSBEndian, (unsigned int) (image->x_resolution*2.54*
+          WriteProfileLong(MSBEndian,(unsigned int) (image->x_resolution*2.54*
             65536.0),p);
         else
-          WriteProfileLong(MSBEndian, (unsigned int) (image->x_resolution*
+          WriteProfileLong(MSBEndian,(unsigned int) (image->x_resolution*
             65536.0),p);
         WriteProfileShort(MSBEndian,(unsigned short) image->units,p+4);
         if (image->units == PixelsPerCentimeterResolution)
-          WriteProfileLong(MSBEndian, (unsigned int) (image->y_resolution*2.54*
+          WriteProfileLong(MSBEndian,(unsigned int) (image->y_resolution*2.54*
             65536.0),p+8);
         else
-          WriteProfileLong(MSBEndian, (unsigned int) (image->y_resolution*
+          WriteProfileLong(MSBEndian,(unsigned int) (image->y_resolution*
             65536.0),p+8);
         WriteProfileShort(MSBEndian,(unsigned short) image->units,p+12);
       }
@@ -2041,6 +2084,9 @@ static MagickBooleanType SyncExifProfile(Image *image, StringInfo *profile)
     number_entries=ReadProfileShort(endian,directory);
     for ( ; entry < number_entries; entry++)
     {
+      int
+        components;
+
       register unsigned char
         *p,
         *q;
@@ -2049,7 +2095,6 @@ static MagickBooleanType SyncExifProfile(Image *image, StringInfo *profile)
         number_bytes;
 
       ssize_t
-        components,
         format,
         tag_value;
 
@@ -2063,7 +2108,7 @@ static MagickBooleanType SyncExifProfile(Image *image, StringInfo *profile)
       format=(ssize_t) ReadProfileShort(endian,q+2);
       if ((format < 0) || ((format-1) >= EXIF_NUM_FORMATS))
         break;
-      components=(ssize_t) ReadProfileLong(endian,q+4);
+      components=(int) ReadProfileLong(endian,q+4);
       if (components < 0)
         break;  /* corrupt EXIF */
       number_bytes=(size_t) components*format_bytes[format];
