@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
 
-    http://www.imagemagick.org/script/license.php
+    https://www.imagemagick.org/script/license.php
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -18,7 +18,10 @@
 #ifndef MAGICKCORE_QUANTUM_PRIVATE_H
 #define MAGICKCORE_QUANTUM_PRIVATE_H
 
+#include "magick/memory_.h"
 #include "magick/cache.h"
+#include "magick/image-private.h"
+#include "magick/pixel-accessor.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -66,7 +69,7 @@ struct _QuantumInfo
   size_t
     number_threads;
 
-  unsigned char
+  MemoryInfo
     **pixels;
 
   size_t
@@ -93,8 +96,15 @@ static inline MagickSizeType GetQuantumRange(const size_t depth)
   MagickSizeType
     one;
 
+  size_t
+    max_depth;
+
+  if (depth == 0)
+    return(0);
   one=1;
-  return((MagickSizeType) ((one << (depth-1))+((one << (depth-1))-1)));
+  max_depth=8*sizeof(MagickSizeType);
+  return((MagickSizeType) ((one << (MagickMin(depth,max_depth)-1))+
+    ((one << (MagickMin(depth,max_depth)-1))-1)));
 }
 
 static inline float HalfToSinglePrecision(const unsigned short half)
@@ -274,9 +284,11 @@ static inline Quantum ScaleAnyToQuantum(const QuantumAny quantum,
   if (quantum > range)
     return(QuantumRange);
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
-  return((Quantum) (((MagickRealType) QuantumRange*quantum)/range+0.5));
+  return((Quantum) (((MagickRealType) QuantumRange*quantum)*
+    PerceptibleReciprocal((double) range)+0.5));
 #else
-  return((Quantum) (((MagickRealType) QuantumRange*quantum)/range));
+  return((Quantum) (((MagickRealType) QuantumRange*quantum)*
+    PerceptibleReciprocal((double) range)));
 #endif
 }
 
@@ -295,7 +307,7 @@ static inline Quantum ScaleCharToQuantum(const unsigned char value)
 static inline Quantum ScaleLongToQuantum(const unsigned int value)
 {
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
-  return((Quantum) ((value+8421504UL)/16843009UL));
+  return((Quantum) ((value)/16843009UL));
 #else
   return((Quantum) (value/16843009.0));
 #endif
@@ -374,8 +386,7 @@ static inline Quantum ScaleCharToQuantum(const unsigned char value)
 static inline Quantum ScaleLongToQuantum(const unsigned int value)
 {
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
-  return((Quantum) ((value+MagickULLConstant(32768))/
-    MagickULLConstant(65537)));
+  return((Quantum) ((value)/MagickULLConstant(65537)));
 #else
   return((Quantum) (value/65537.0));
 #endif
