@@ -17,13 +17,13 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -70,7 +70,7 @@
   Define declarations.
 */
 #define MagickPathTemplate "XXXXXXXXXXXX"
-
+
 /*
   Typedef declarations.
 */
@@ -79,6 +79,7 @@ typedef struct _ResourceInfo
   MagickOffsetType
     width,
     height,
+    list_length,
     area,
     memory,
     map,
@@ -91,6 +92,7 @@ typedef struct _ResourceInfo
   MagickSizeType
     width_limit,
     height_limit,
+    list_length_limit,
     area_limit,
     memory_limit,
     map_limit,
@@ -112,6 +114,7 @@ static ResourceInfo
   {
     MagickULLConstant(0),              /* initial width */
     MagickULLConstant(0),              /* initial height */
+    MagickULLConstant(0),              /* initial list length */
     MagickULLConstant(0),              /* initial area */
     MagickULLConstant(0),              /* initial memory */
     MagickULLConstant(0),              /* initial map */
@@ -122,6 +125,7 @@ static ResourceInfo
     MagickULLConstant(0),              /* initial time */
     (INT_MAX/(5*sizeof(Quantum))),     /* width limit */
     (INT_MAX/(5*sizeof(Quantum))),     /* height limit */
+    MagickResourceInfinity,            /* list length limit */
     MagickULLConstant(3072)*1024*1024, /* area limit */
     MagickULLConstant(1536)*1024*1024, /* memory limit */
     MagickULLConstant(3072)*1024*1024, /* map limit */
@@ -174,150 +178,224 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     resource_request[MaxTextExtent];
 
   MagickBooleanType
+    logging,
     status;
 
+  MagickOffsetType
+    request;
+
   MagickSizeType
     limit;
 
+  request=(MagickOffsetType) size;
+  if (request < 0)
+    return(MagickFalse);
   status=MagickFalse;
-  (void) FormatMagickSize(size,MagickFalse,resource_request);
+  logging=IsEventLogging();
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     ActivateSemaphoreInfo(&resource_semaphore);
   LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
-    case WidthResource:
+    case AreaResource:
     {
-      resource_info.width=(MagickOffsetType) size;
-      limit=resource_info.width_limit;
-      status=(resource_info.width_limit == MagickResourceInfinity) ||
-        (size < limit) ? MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.width,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize(resource_info.width_limit,MagickFalse,
-        resource_limit);
+      resource_info.area=(MagickOffsetType) size;
+      limit=resource_info.area_limit;
+      if ((limit == MagickResourceInfinity) || (size < limit))
+        status=MagickTrue;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,resource_request);
+          (void) FormatMagickSize(size,MagickFalse,resource_current);
+          (void) FormatMagickSize(limit,MagickFalse,resource_limit);
+        }
+      break;
+    }
+    case DiskResource:
+    {
+      limit=resource_info.disk_limit;
+      if ((resource_info.disk+request) > resource_info.disk)
+        {
+          resource_info.disk+=request;
+          if ((limit == MagickResourceInfinity) ||
+              (resource_info.disk < (MagickOffsetType) limit))
+            status=MagickTrue;
+          else
+            resource_info.disk-=request;
+        }
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickTrue,resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.disk,
+            MagickTrue,resource_current);
+          (void) FormatMagickSize(limit,MagickTrue,resource_limit);
+        }
+      break;
+    }
+    case FileResource:
+    {
+      limit=resource_info.file_limit;
+      if ((resource_info.file+request) > resource_info.file)
+        {
+          resource_info.file+=request;
+          if ((limit == MagickResourceInfinity) ||
+              (resource_info.file < (MagickOffsetType) limit))
+            status=MagickTrue;
+        }
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.file,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize(limit,MagickFalse,resource_limit);
+        }
       break;
     }
     case HeightResource:
     {
       resource_info.height=(MagickOffsetType) size;
       limit=resource_info.height_limit;
-      status=(resource_info.height_limit == MagickResourceInfinity) ||
-        (size < limit) ? MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.height,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize(resource_info.height_limit,MagickFalse,
-        resource_limit);
+      if ((limit == MagickResourceInfinity) || (size < limit))
+        status=MagickTrue;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,resource_request);
+          (void) FormatMagickSize(size,MagickFalse,resource_current);
+          (void) FormatMagickSize(limit,MagickFalse,resource_limit);
+        }
       break;
     }
-    case AreaResource:
+    case ListLengthResource:
     {
-      resource_info.area=(MagickOffsetType) size;
-      limit=resource_info.area_limit;
-      status=(resource_info.area_limit == MagickResourceInfinity) ||
-        (size < limit) ? MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.area,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize(resource_info.area_limit,MagickFalse,
-        resource_limit);
-      break;
-    }
-    case MemoryResource:
-    {
-      resource_info.memory+=size;
-      limit=resource_info.memory_limit;
-      status=(resource_info.memory_limit == MagickResourceInfinity) ||
-        ((MagickSizeType) resource_info.memory < limit) ?
-        MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.memory,MagickTrue,
-        resource_current);
-      (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,
-        resource_limit);
+      resource_info.list_length=(MagickOffsetType) size;
+      limit=resource_info.list_length_limit;
+      if ((limit == MagickResourceInfinity) || (size < limit))
+        status=MagickTrue;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,resource_request);
+          (void) FormatMagickSize(size,MagickFalse,resource_current);
+          (void) FormatMagickSize(limit,MagickFalse,resource_limit);
+        }
       break;
     }
     case MapResource:
     {
-      resource_info.map+=size;
       limit=resource_info.map_limit;
-      status=(resource_info.map_limit == MagickResourceInfinity) ||
-        ((MagickSizeType) resource_info.map < limit) ?
-        MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.map,MagickTrue,
-        resource_current);
-      (void) FormatMagickSize(resource_info.map_limit,MagickTrue,
-        resource_limit);
+      if ((resource_info.map+request) > resource_info.map)
+        {
+          resource_info.map+=request;
+          if ((limit == MagickResourceInfinity) ||
+              (resource_info.map < (MagickOffsetType) limit))
+            status=MagickTrue;
+          else
+            resource_info.map-=request;
+        }
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickTrue,resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.map,
+            MagickTrue,resource_current);
+          (void) FormatMagickSize(limit,MagickTrue,resource_limit);
+        }
       break;
     }
-    case DiskResource:
+    case MemoryResource:
     {
-      resource_info.disk+=size;
-      limit=resource_info.disk_limit;
-      status=(resource_info.disk_limit == MagickResourceInfinity) ||
-        ((MagickSizeType) resource_info.disk < limit) ?
-        MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.disk,MagickTrue,
-        resource_current);
-      (void) FormatMagickSize(resource_info.disk_limit,MagickTrue,
-        resource_limit);
-      break;
-    }
-    case FileResource:
-    {
-      resource_info.file+=size;
-      limit=resource_info.file_limit;
-      status=(resource_info.file_limit == MagickResourceInfinity) ||
-        ((MagickSizeType) resource_info.file < limit) ?
-        MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.file,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize((MagickSizeType) resource_info.file_limit,
-        MagickFalse,resource_limit);
+      limit=resource_info.memory_limit;
+      if ((resource_info.memory+request) > resource_info.memory)
+        {
+          resource_info.memory+=request;
+          if ((limit == MagickResourceInfinity) ||
+              (resource_info.memory < (MagickOffsetType) limit))
+            status=MagickTrue;
+          else
+            resource_info.memory-=request;
+        }
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickTrue,resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.memory,
+            MagickTrue,resource_current);
+          (void) FormatMagickSize(limit,MagickTrue,resource_limit);
+        }
       break;
     }
     case ThreadResource:
     {
       limit=resource_info.thread_limit;
-      status=(resource_info.thread_limit == MagickResourceInfinity) ||
-        ((MagickSizeType) resource_info.thread < limit) ? MagickTrue :
-        MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.thread,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize((MagickSizeType) resource_info.thread_limit,
-        MagickFalse,resource_limit);
+      if ((limit == MagickResourceInfinity) ||
+          (resource_info.thread < (MagickOffsetType) limit))
+        status=MagickTrue;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.thread,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize(limit,MagickFalse,resource_limit);
+        }
       break;
     }
     case ThrottleResource:
     {
       limit=resource_info.throttle_limit;
-      status=(resource_info.throttle_limit == MagickResourceInfinity) ||
-        ((MagickSizeType) resource_info.throttle < limit) ? MagickTrue :
-        MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.throttle,
-        MagickFalse,resource_current);
-      (void) FormatMagickSize((MagickSizeType) resource_info.throttle_limit,
-        MagickFalse,resource_limit);
+      if ((limit == MagickResourceInfinity) ||
+          (resource_info.throttle < (MagickOffsetType) limit))
+        status=MagickTrue;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.throttle,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize(limit,MagickFalse,resource_limit);
+        }
       break;
     }
     case TimeResource:
     {
-      resource_info.time+=size;
       limit=resource_info.time_limit;
-      status=(resource_info.time_limit == MagickResourceInfinity) ||
-        ((MagickSizeType) resource_info.time < limit) ?
-        MagickTrue : MagickFalse;
-      (void) FormatMagickSize((MagickSizeType) resource_info.time,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize((MagickSizeType) resource_info.time_limit,
-        MagickFalse,resource_limit);
+      if ((resource_info.time+request) > resource_info.time)
+        {
+          resource_info.time+=request;
+          if ((limit == MagickResourceInfinity) ||
+              (resource_info.time < (MagickOffsetType) limit))
+            status=MagickTrue;
+          else
+            resource_info.time-=request;
+        }
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,resource_request);
+          (void) FormatMagickSize((MagickSizeType) resource_info.time,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize(limit,MagickFalse,resource_limit);
+        }
+      break;
+    }
+    case WidthResource:
+    {
+      resource_info.width=(MagickOffsetType) size;
+      limit=resource_info.width_limit;
+      if ((limit == MagickResourceInfinity) || (size < limit))
+        status=MagickTrue;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize(size,MagickFalse,resource_request);
+          (void) FormatMagickSize(size,MagickFalse,resource_current);
+          (void) FormatMagickSize(limit,MagickFalse,resource_limit);
+        }
       break;
     }
     default:
       break;
   }
   UnlockSemaphoreInfo(resource_semaphore);
-  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
-    CommandOptionToMnemonic(MagickResourceOptions,(ssize_t) type),
-    resource_request,resource_current,resource_limit);
+  if (logging != MagickFalse)
+    {
+      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
+        CommandOptionToMnemonic(MagickResourceOptions,(ssize_t) type),
+        resource_request,resource_current,resource_limit);
+    }
   return(status);
 }
 
@@ -435,7 +513,7 @@ MagickExport MagickBooleanType GetPathTemplate(char *path)
 #endif
   if (directory == (char *) NULL)
     return(MagickTrue);
-  value=GetPolicyValue("temporary-path");
+  value=GetPolicyValue("resource:temporary-path");
   if (value != (char *) NULL)
     {
       (void) CloneString(&directory,value);
@@ -518,6 +596,8 @@ MagickExport int AcquireUniqueFileResource(char *path)
   file=(-1);
   for (i=0; i < (ssize_t) TMP_MAX; i++)
   {
+    register ssize_t
+      j;
     /*
       Get temporary pathname.
     */
@@ -525,9 +605,9 @@ MagickExport int AcquireUniqueFileResource(char *path)
     key=GetRandomKey(random_info,6);
     p=path+strlen(path)-strlen(MagickPathTemplate);
     datum=GetStringInfoDatum(key);
-    for (i=0; i < (ssize_t) GetStringInfoLength(key); i++)
+    for (j=0; j < (ssize_t) GetStringInfoLength(key); j++)
     {
-      c=(int) (datum[i] & 0x3f);
+      c=(int) (datum[j] & 0x3f);
       *p++=portable_filename[c];
     }
     key=DestroyStringInfo(key);
@@ -547,9 +627,9 @@ MagickExport int AcquireUniqueFileResource(char *path)
     key=GetRandomKey(random_info,strlen(MagickPathTemplate));
     p=path+strlen(path)-strlen(MagickPathTemplate);
     datum=GetStringInfoDatum(key);
-    for (i=0; i < (ssize_t) GetStringInfoLength(key); i++)
+    for (j=0; j < (ssize_t) GetStringInfoLength(key); j++)
     {
-      c=(int) (datum[i] & 0x3f);
+      c=(int) (datum[j] & 0x3f);
       *p++=portable_filename[c];
     }
     key=DestroyStringInfo(key);
@@ -604,31 +684,11 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
   LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
-    case WidthResource:
-    {
-      resource=(MagickSizeType) resource_info.width;
-      break;
-    }
-    case HeightResource:
-    {
-      resource=(MagickSizeType) resource_info.height;
-      break;
-    }
     case AreaResource:
     {
       resource=(MagickSizeType) resource_info.area;
       break;
     }
-    case MemoryResource:
-    {
-      resource=(MagickSizeType) resource_info.memory;
-      break;
-    }
-    case MapResource:
-    {
-      resource=(MagickSizeType) resource_info.map;
-      break;
-    }
     case DiskResource:
     {
       resource=(MagickSizeType) resource_info.disk;
@@ -639,6 +699,26 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
       resource=(MagickSizeType) resource_info.file;
       break;
     }
+    case HeightResource:
+    {
+      resource=(MagickSizeType) resource_info.height;
+      break;
+    }
+    case ListLengthResource:
+    {
+      resource=(MagickSizeType) resource_info.list_length;
+      break;
+    }
+    case MapResource:
+    {
+      resource=(MagickSizeType) resource_info.map;
+      break;
+    }
+    case MemoryResource:
+    {
+      resource=(MagickSizeType) resource_info.memory;
+      break;
+    }
     case ThreadResource:
     {
       resource=(MagickSizeType) resource_info.thread;
@@ -654,6 +734,11 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
       resource=(MagickSizeType) resource_info.time;
       break;
     }
+    case WidthResource:
+    {
+      resource=(MagickSizeType) resource_info.width;
+      break;
+    }
     default:
       break;
   }
@@ -694,31 +779,11 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
   LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
-    case WidthResource:
-    {
-      resource=resource_info.width_limit;
-      break;
-    }
-    case HeightResource:
-    {
-      resource=resource_info.height_limit;
-      break;
-    }
     case AreaResource:
     {
       resource=resource_info.area_limit;
       break;
     }
-    case MemoryResource:
-    {
-      resource=resource_info.memory_limit;
-      break;
-    }
-    case MapResource:
-    {
-      resource=resource_info.map_limit;
-      break;
-    }
     case DiskResource:
     {
       resource=resource_info.disk_limit;
@@ -729,6 +794,26 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
       resource=resource_info.file_limit;
       break;
     }
+    case HeightResource:
+    {
+      resource=resource_info.height_limit;
+      break;
+    }
+    case ListLengthResource:
+    {
+      resource=resource_info.list_length_limit;
+      break;
+    }
+    case MemoryResource:
+    {
+      resource=resource_info.memory_limit;
+      break;
+    }
+    case MapResource:
+    {
+      resource=resource_info.map_limit;
+      break;
+    }
     case ThreadResource:
     {
       resource=resource_info.thread_limit;
@@ -744,6 +829,11 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
       resource=resource_info.time_limit;
       break;
     }
+    case WidthResource:
+    {
+      resource=resource_info.width_limit;
+      break;
+    }
     default:
       break;
   }
@@ -836,6 +926,7 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
     area_limit[MaxTextExtent],
     disk_limit[MaxTextExtent],
     height_limit[MaxTextExtent],
+    list_length_limit[MaxTextExtent],
     map_limit[MaxTextExtent],
     memory_limit[MaxTextExtent],
     time_limit[MaxTextExtent],
@@ -850,6 +941,8 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
   LockSemaphoreInfo(resource_semaphore);
   (void) FormatPixelSize(resource_info.width_limit,MagickFalse,width_limit);
   (void) FormatPixelSize(resource_info.height_limit,MagickFalse,height_limit);
+  (void) FormatPixelSize(resource_info.list_length_limit,MagickFalse,
+    list_length_limit);
   (void) FormatPixelSize(resource_info.area_limit,MagickFalse,area_limit);
   (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,memory_limit);
   (void) FormatMagickSize(resource_info.map_limit,MagickTrue,map_limit);
@@ -863,6 +956,7 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
   (void) FormatLocaleFile(file,"Resource limits:\n");
   (void) FormatLocaleFile(file,"  Width: %s\n",width_limit);
   (void) FormatLocaleFile(file,"  Height: %s\n",height_limit);
+  (void) FormatLocaleFile(file,"  List length: %s\n",list_length_limit);
   (void) FormatLocaleFile(file,"  Area: %s\n",area_limit);
   (void) FormatLocaleFile(file,"  Memory: %s\n",memory_limit);
   (void) FormatLocaleFile(file,"  Map: %s\n",map_limit);
@@ -912,107 +1006,163 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     resource_limit[MaxTextExtent],
     resource_request[MaxTextExtent];
 
-  (void) FormatMagickSize(size,MagickFalse,resource_request);
+  MagickBooleanType
+    logging;
+
+  logging=IsEventLogging();
+  if (logging != MagickFalse)
+    (void) FormatMagickSize(size,MagickFalse,resource_request);
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     ActivateSemaphoreInfo(&resource_semaphore);
   LockSemaphoreInfo(resource_semaphore);
   switch (type)
   {
-    case WidthResource:
-    {
-      resource_info.width=(MagickOffsetType) size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.width,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize(resource_info.width_limit,MagickFalse,
-        resource_limit);
-      break;
-    }
-    case HeightResource:
-    {
-      resource_info.height=(MagickOffsetType) size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.height,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize(resource_info.height_limit,MagickFalse,
-        resource_limit);
-      break;
-    }
     case AreaResource:
     {
       resource_info.area=(MagickOffsetType) size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.area,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize(resource_info.area_limit,MagickFalse,
-        resource_limit);
-      break;
-    }
-    case MemoryResource:
-    {
-      resource_info.memory-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.memory,
-        MagickTrue,resource_current);
-      (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,
-        resource_limit);
-      break;
-    }
-    case MapResource:
-    {
-      resource_info.map-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.map,MagickTrue,
-        resource_current);
-      (void) FormatMagickSize(resource_info.map_limit,MagickTrue,
-        resource_limit);
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.area,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize(resource_info.area_limit,MagickFalse,
+            resource_limit);
+        }
       break;
     }
     case DiskResource:
     {
       resource_info.disk-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.disk,MagickTrue,
-        resource_current);
-      (void) FormatMagickSize(resource_info.disk_limit,MagickTrue,
-        resource_limit);
+      assert(resource_info.disk >= 0);
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.disk,
+            MagickTrue,resource_current);
+          (void) FormatMagickSize(resource_info.disk_limit,MagickTrue,
+            resource_limit);
+        }
       break;
     }
     case FileResource:
     {
       resource_info.file-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.file,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize((MagickSizeType) resource_info.file_limit,
-        MagickFalse,resource_limit);
+      assert(resource_info.file >= 0);
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.file,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize((MagickSizeType) resource_info.file_limit,
+            MagickFalse,resource_limit);
+        }
+      break;
+    }
+    case HeightResource:
+    {
+      resource_info.height=(MagickOffsetType) size;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.height,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize(resource_info.height_limit,MagickFalse,
+            resource_limit);
+        }
+      break;
+    }
+    case ListLengthResource:
+    {
+      resource_info.list_length=(MagickOffsetType) size;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.list_length,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize(resource_info.list_length_limit,MagickFalse,
+            resource_limit);
+        }
+      break;
+    }
+    case MapResource:
+    {
+      resource_info.map-=size;
+      assert(resource_info.map >= 0);
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.map,
+            MagickTrue,resource_current);
+          (void) FormatMagickSize(resource_info.map_limit,MagickTrue,
+            resource_limit);
+        }
+      break;
+    }
+    case MemoryResource:
+    {
+      resource_info.memory-=size;
+      assert(resource_info.memory >= 0);
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.memory,
+            MagickTrue,resource_current);
+          (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,
+            resource_limit);
+        }
       break;
     }
     case ThreadResource:
     {
-      (void) FormatMagickSize((MagickSizeType) resource_info.thread,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize((MagickSizeType) resource_info.thread_limit,
-        MagickFalse,resource_limit);
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.thread,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize((MagickSizeType) resource_info.thread_limit,
+            MagickFalse,resource_limit);
+        }
       break;
     }
     case ThrottleResource:
     {
-      (void) FormatMagickSize((MagickSizeType) resource_info.throttle,
-        MagickFalse,resource_current);
-      (void) FormatMagickSize((MagickSizeType) resource_info.throttle_limit,
-        MagickFalse,resource_limit);
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.throttle,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize((MagickSizeType) resource_info.throttle_limit,
+            MagickFalse,resource_limit);
+        }
       break;
     }
     case TimeResource:
     {
       resource_info.time-=size;
-      (void) FormatMagickSize((MagickSizeType) resource_info.time,MagickFalse,
-        resource_current);
-      (void) FormatMagickSize((MagickSizeType) resource_info.time_limit,
-        MagickFalse,resource_limit);
+      assert(resource_info.time >= 0);
+      if (logging != MagickFalse)
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.time,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize((MagickSizeType) resource_info.time_limit,
+            MagickFalse,resource_limit);
+        }
+      break;
+    }
+    case WidthResource:
+    {
+      resource_info.width=(MagickOffsetType) size;
+      if (logging != MagickFalse)
+        {
+          (void) FormatMagickSize((MagickSizeType) resource_info.width,
+            MagickFalse,resource_current);
+          (void) FormatMagickSize(resource_info.width_limit,MagickFalse,
+            resource_limit);
+        }
       break;
     }
     default:
       break;
   }
   UnlockSemaphoreInfo(resource_semaphore);
-  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
-    CommandOptionToMnemonic(MagickResourceOptions,(ssize_t) type),
-      resource_request,resource_current,resource_limit);
+  if (logging != MagickFalse)
+    {
+      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
+        CommandOptionToMnemonic(MagickResourceOptions,(ssize_t) type),
+          resource_request,resource_current,resource_limit);
+    }
 }
 
 /*
@@ -1055,8 +1205,9 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
     status=DeleteNodeFromSplayTree(temporary_resources, (const void *) path);
   UnlockSemaphoreInfo(resource_semaphore);
   (void) CopyMagickString(cache_path,path,MaxTextExtent);
-  AppendImageFormat("cache",cache_path);
-  (void) ShredFile(cache_path);
+    AppendImageFormat("cache",cache_path);
+  if (access_utf8(cache_path,F_OK) == 0)
+    (void) ShredFile(cache_path);
   if (status == MagickFalse)
     status=ShredFile(path);
   return(status);
@@ -1218,6 +1369,14 @@ MagickExport MagickBooleanType ResourceComponentGenesis(void)
       (void) SetMagickResourceLimit(TimeResource,StringToSizeType(limit,100.0));
       limit=DestroyString(limit);
     }
+  (void) SetMagickResourceLimit(ListLengthResource,MagickResourceInfinity);
+  limit=GetEnvironmentValue("MAGICK_LIST_LENGTH_LIMIT");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(ListLengthResource,
+        StringToSizeType(limit,100.0));
+      limit=DestroyString(limit);
+    }
   return(MagickTrue);
 }
 
@@ -1284,109 +1443,135 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
   char
     *value;
 
+  MagickBooleanType
+    status;
+
+  status=MagickTrue;
   if (resource_semaphore == (SemaphoreInfo *) NULL)
     ActivateSemaphoreInfo(&resource_semaphore);
   LockSemaphoreInfo(resource_semaphore);
   value=(char *) NULL;
   switch (type)
   {
-    case WidthResource:
-    {
-      resource_info.width_limit=limit;
-      value=GetPolicyValue("width");
-      if (value != (char *) NULL)
-        resource_info.width_limit=MagickMin(limit,StringToSizeType(value,
-          100.0));
-      break;
-    }
-    case HeightResource:
-    {
-      resource_info.height_limit=limit;
-      value=GetPolicyValue("height");
-      if (value != (char *) NULL)
-        resource_info.height_limit=MagickMin(limit,StringToSizeType(value,
-          100.0));
-      break;
-    }
     case AreaResource:
     {
-      resource_info.area_limit=limit;
-      value=GetPolicyValue("area");
-      if (value != (char *) NULL)
+      value=GetPolicyValue("resource:area");
+      if (value == (char *) NULL)
+        resource_info.area_limit=limit;
+      else
         resource_info.area_limit=MagickMin(limit,StringToSizeType(value,100.0));
       break;
     }
-    case MemoryResource:
-    {
-      resource_info.memory_limit=limit;
-      value=GetPolicyValue("memory");
-      if (value != (char *) NULL)
-        resource_info.memory_limit=MagickMin(limit,StringToSizeType(value,
-          100.0));
-      break;
-    }
-    case MapResource:
-    {
-      resource_info.map_limit=limit;
-      value=GetPolicyValue("map");
-      if (value != (char *) NULL)
-        resource_info.map_limit=MagickMin(limit,StringToSizeType(value,100.0));
-      break;
-    }
     case DiskResource:
     {
-      resource_info.disk_limit=limit;
-      value=GetPolicyValue("disk");
-      if (value != (char *) NULL)
+      value=GetPolicyValue("resource:disk");
+      if (value == (char *) NULL)
+        resource_info.disk_limit=limit;
+      else
         resource_info.disk_limit=MagickMin(limit,StringToSizeType(value,100.0));
       break;
     }
     case FileResource:
     {
-      resource_info.file_limit=limit;
-      value=GetPolicyValue("file");
-      if (value != (char *) NULL)
+      value=GetPolicyValue("resource:file");
+      if (value == (char *) NULL)
+        resource_info.file_limit=limit;
+      else
         resource_info.file_limit=MagickMin(limit,StringToSizeType(value,100.0));
       break;
     }
+    case HeightResource:
+    {
+      value=GetPolicyValue("resource:height");
+      if (value == (char *) NULL)
+        resource_info.height_limit=limit;
+      else
+        resource_info.height_limit=MagickMin(limit,StringToSizeType(value,
+          100.0));
+      break;
+    }
+    case ListLengthResource:
+    {
+      value=GetPolicyValue("resource:list-length");
+      if (value == (char *) NULL)
+        resource_info.list_length_limit=limit;
+      else
+        resource_info.list_length_limit=MagickMin(limit,
+          StringToSizeType(value,100.0));
+      break;
+    }
+    case MapResource:
+    {
+      value=GetPolicyValue("resource:map");
+      if (value == (char *) NULL)
+        resource_info.map_limit=limit;
+      else
+        resource_info.map_limit=MagickMin(limit,StringToSizeType(value,100.0));
+      break;
+    }
+    case MemoryResource:
+    {
+      value=GetPolicyValue("resource:memory");
+      if (value == (char *) NULL)
+        resource_info.memory_limit=limit;
+      else
+        resource_info.memory_limit=MagickMin(limit,StringToSizeType(value,
+          100.0));
+      break;
+    }
     case ThreadResource:
     {
-      resource_info.thread_limit=limit;
-      value=GetPolicyValue("thread");
-      if (value != (char *) NULL)
+      value=GetPolicyValue("resource:thread");
+      if (value == (char *) NULL)
+        resource_info.thread_limit=limit;
+      else
         resource_info.thread_limit=MagickMin(limit,StringToSizeType(value,
           100.0));
       if (resource_info.thread_limit > GetOpenMPMaximumThreads())
         resource_info.thread_limit=GetOpenMPMaximumThreads();
+      else
+        if (resource_info.thread_limit == 0)
+          resource_info.thread_limit=1;
       break;
     }
     case ThrottleResource:
     {
-      resource_info.throttle_limit=limit;
-      value=GetPolicyValue("throttle");
-      if (value != (char *) NULL)
-        resource_info.throttle_limit=MagickMin(limit,StringToSizeType(value,
+      value=GetPolicyValue("resource:throttle");
+      if (value == (char *) NULL)
+        resource_info.throttle_limit=limit;
+      else
+        resource_info.throttle_limit=MagickMax(limit,StringToSizeType(value,
           100.0));
-      if (resource_info.throttle_limit > GetOpenMPMaximumThreads())
-        resource_info.throttle_limit=GetOpenMPMaximumThreads();
-      else if (resource_info.thread_limit == 0)
-        resource_info.thread_limit=1;
       break;
     }
     case TimeResource:
     {
-      resource_info.time_limit=limit;
-      value=GetPolicyValue("time");
-      if (value != (char *) NULL)
+      value=GetPolicyValue("resource:time");
+      if (value == (char *) NULL)
+        resource_info.time_limit=limit;
+      else
         resource_info.time_limit=MagickMin(limit,StringToSizeType(value,100.0));
       ResetPixelCacheEpoch();
       break;
     }
-    default:
+    case WidthResource:
+    {
+      value=GetPolicyValue("resource:width");
+      if (value == (char *) NULL)
+        resource_info.width_limit=limit;
+      else
+        resource_info.width_limit=MagickMin(limit,StringToSizeType(value,
+          100.0));
       break;
+    }
+    default:
+    {
+      status=MagickFalse;
+      break;
+    }
   }
   if (value != (char *) NULL)
     value=DestroyString(value);
   UnlockSemaphoreInfo(resource_semaphore);
-  return(MagickTrue);
+  return(status);
 }
