@@ -17,13 +17,13 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -155,6 +155,9 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
     destination_file,
     source_file;
 
+  MagickBooleanType
+    status;
+
   size_t
     length,
     quantum;
@@ -171,26 +174,38 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
   assert(source != (const char *) NULL);
   assert(destination != (char *) NULL);
 #if defined(MAGICKCORE_HAVE_SYMLINK)
-  (void) AcquireUniqueFilename(destination);
-  (void) RelinquishUniqueFileResource(destination);
-  if (*source == *DirectorySeparator)
-    {
-      if (symlink(source,destination) == 0)
-        return(MagickTrue);
-    }
-  else
-    {
-      char
-        path[MaxTextExtent];
+  {
+    char
+      *passes;
 
-      *path='\0';
-      if (getcwd(path,MaxTextExtent) == (char *) NULL)
-        return(MagickFalse);
-      (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
-      (void) ConcatenateMagickString(path,source,MaxTextExtent);
-      if (symlink(path,destination) == 0)
-        return(MagickTrue);
-    }
+    (void) AcquireUniqueFilename(destination);
+    (void) RelinquishUniqueFileResource(destination);
+    passes=GetPolicyValue("system:shred");
+    if (passes != (char *) NULL)
+      passes=DestroyString(passes);
+    else
+      {
+        if (*source == *DirectorySeparator)
+          {
+            if (symlink(source,destination) == 0)
+              return(MagickTrue);
+          }
+        else
+          {
+            char
+              path[MaxTextExtent];
+
+            *path='\0';
+            if (getcwd(path,MaxTextExtent) == (char *) NULL)
+              return(MagickFalse);
+            (void) ConcatenateMagickString(path,DirectorySeparator,
+              MaxTextExtent);
+            (void) ConcatenateMagickString(path,source,MaxTextExtent);
+            if (symlink(path,destination) == 0)
+              return(MagickTrue);
+          }
+      }
+  }
 #endif
   destination_file=AcquireUniqueFileResource(destination);
   if (destination_file == -1)
@@ -213,6 +228,7 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
       (void) RelinquishUniqueFileResource(destination);
       return(MagickFalse);
     }
+  status=MagickTrue;
   for (length=0; ; )
   {
     count=(ssize_t) read(source_file,buffer,quantum);
@@ -222,17 +238,15 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
     count=(ssize_t) write(destination_file,buffer,length);
     if ((size_t) count != length)
       {
-        (void) close(destination_file);
-        (void) close(source_file);
-        buffer=(unsigned char *) RelinquishMagickMemory(buffer);
         (void) RelinquishUniqueFileResource(destination);
-        return(MagickFalse);
+        status=MagickFalse;
+        break;
       }
   }
   (void) close(destination_file);
   (void) close(source_file);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
-  return(MagickTrue);
+  return(status);
 }
 
 /*
@@ -1039,14 +1053,12 @@ MagickExport MagickBooleanType GetExecutionPath(char *path,const size_t extent)
 #if defined(__GNU__)
   {
     char
-      *program_name,
-      *execution_path;
+      *program_name;
 
     ssize_t
       count;
 
     count=0;
-    execution_path=(char *) NULL;
     program_name=program_invocation_name;
     if (*program_invocation_name != '/')
       {
@@ -1063,13 +1075,14 @@ MagickExport MagickBooleanType GetExecutionPath(char *path,const size_t extent)
       }
     if (count != -1)
       {
-        execution_path=realpath(program_name,NULL);
-        if (execution_path != (char *) NULL)
+        char
+          execution_path[PATH_MAX+1];
+
+        if (realpath(program_name,execution_path) != (char *) NULL)
           (void) CopyMagickString(path,execution_path,extent);
       }
     if (program_name != program_invocation_name)
       program_name=(char *) RelinquishMagickMemory(program_name);
-    execution_path=(char *) RelinquishMagickMemory(execution_path);
   }
 #endif
 #if defined(__OpenBSD__)
@@ -1110,13 +1123,13 @@ MagickExport ssize_t GetMagickPageSize(void)
     return(page_size);
 #if defined(MAGICKCORE_HAVE_SYSCONF) && defined(_SC_PAGE_SIZE)
   page_size=(ssize_t) sysconf(_SC_PAGE_SIZE);
-#else
-#if defined(MAGICKCORE_HAVE_GETPAGESIZE)
+#elif defined(MAGICKCORE_HAVE_GETPAGESIZE)
   page_size=(ssize_t) getpagesize();
-#endif
+#elif defined(MAGICKCORE_WINDOWS_SUPPORT)
+  page_size=NTGetPageSize();
 #endif
   if (page_size <= 0)
-    page_size=16384;
+    page_size=4096;
   return(page_size);
 }
 
@@ -1155,7 +1168,7 @@ MagickExport MagickBooleanType GetPathAttributes(const char *path,
       errno=EINVAL;
       return(MagickFalse);
     }
-  (void) ResetMagickMemory(attributes,0,sizeof(struct stat));
+  (void) memset(attributes,0,sizeof(struct stat));
   status=stat_utf8(path,(struct stat *) attributes) == 0 ? MagickTrue :
     MagickFalse;
   return(status);
@@ -1227,7 +1240,7 @@ MagickExport void GetPathComponent(const char *path,PathType type,
         if (*p == '\0')
           break;
       }
-    if ((*p == ':') && (IsPathDirectory(path) < 0) &&
+    if ((p != component) && (*p == ':') && (IsPathDirectory(path) < 0) &&
         (IsPathAccessible(path) == MagickFalse))
       {
         /*
@@ -1280,16 +1293,19 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     }
     case RootPath:
     {
-      for (p=component+(strlen(component)-1); p > component; p--)
-      {
-        if (IsBasenameSeparator(*p) != MagickFalse)
+      if (*component != '\0')
+        {
+          for (p=component+(strlen(component)-1); p > component; p--)
+          {
+            if (IsBasenameSeparator(*p) != MagickFalse)
+              break;
+            if (*p == '.')
+              break;
+          }
+          if (*p == '.')
+            *p='\0';
           break;
-        if (*p == '.')
-          break;
-      }
-      if (*p == '.')
-        *p='\0';
-      break;
+        }
     }
     case HeadPath:
     {
@@ -1299,28 +1315,27 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     case TailPath:
     {
       if (IsBasenameSeparator(*p) != MagickFalse)
-        (void) CopyMagickMemory((unsigned char *) component,
-          (const unsigned char *) (p+1),strlen(p+1)+1);
+        (void) CopyMagickString(component,p+1,MagickPathExtent);
       break;
     }
     case BasePath:
     {
       if (IsBasenameSeparator(*p) != MagickFalse)
         (void) CopyMagickString(component,p+1,MaxTextExtent);
-      for (p=component+(strlen(component)-1); p > component; p--)
-        if (*p == '.')
-          {
-            *p='\0';
-            break;
-          }
+      if (*component != '\0')
+        for (p=component+(strlen(component)-1); p > component; p--)
+          if (*p == '.')
+            {
+              *p='\0';
+              break;
+            }
       break;
     }
     case ExtensionPath:
     {
       if (IsBasenameSeparator(*p) != MagickFalse)
         (void) CopyMagickString(component,p+1,MaxTextExtent);
-      p=component;
-      if (*p != '\0')
+      if (*component != '\0')
         for (p=component+strlen(component)-1; p > component; p--)
           if (*p == '.')
             break;
@@ -1545,17 +1560,6 @@ static int FileCompare(const void *x,const void *y)
 }
 #endif
 
-static inline int MagickReadDirectory(DIR *directory,struct dirent *entry,
-  struct dirent **result)
-{
-  errno=0;
-  entry=readdir(directory);
-  *result=entry;
-  if ((entry == (struct dirent *) NULL) && (errno != 0))
-    return(-1);
-  return(0);
-}
-
 MagickExport char **ListFiles(const char *directory,const char *pattern,
   size_t *number_entries)
 {
@@ -1807,7 +1811,9 @@ MagickPrivate MagickBooleanType ShredFile(const char *path)
 
   if ((path == (const char *) NULL) || (*path == '\0'))
     return(MagickFalse);
-  passes=GetEnvironmentValue("MAGICK_SHRED_PASSES");
+  passes=GetPolicyValue("system:shred");
+  if (passes == (char *) NULL)
+    passes=GetEnvironmentValue("MAGICK_SHRED_PASSES");
   if (passes == (char *) NULL)
     {
       /*
@@ -1866,7 +1872,7 @@ MagickPrivate MagickBooleanType ShredFile(const char *path)
       if (i == 0)
         ResetStringInfo(key);  /* zero on first pass */
       count=write(file,GetStringInfoDatum(key),(size_t)
-        MagickMin(quantum,length-j));
+        MagickMin((MagickSizeType) quantum,length-j));
       key=DestroyStringInfo(key);
       if (count <= 0)
         {
