@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -26,13 +26,13 @@ extern "C" {
   Define declarations.
 */
 #define MagickPackageName "ImageMagick"
-#define MagickCopyright  "© 1999-2017 ImageMagick Studio LLC"
-#define MagickLibVersion  0x697
-#define MagickLibVersionText  "6.9.7"
-#define MagickLibVersionNumber  3,0,0
-#define MagickLibAddendum  "-4"
-#define MagickLibInterface  3
-#define MagickLibMinInterface  3
+#define MagickCopyright  "© 1999-2018 ImageMagick Studio LLC"
+#define MagickLibVersion  0x69A
+#define MagickLibVersionText  "6.9.10"
+#define MagickLibVersionNumber  6,0,0
+#define MagickLibAddendum  "-8"
+#define MagickLibInterface  6
+#define MagickLibMinInterface  6
 #if defined(_WINDOWS)
 #  if defined(_WIN64)
 #    define MagickPlatform "x64"
@@ -42,16 +42,16 @@ extern "C" {
 #else
 #define MagickPlatform  "x86_64"
 #endif
-#define MagickppLibVersionText  "6.9.7"
-#define MagickppLibVersionNumber  7:0:0
-#define MagickppLibAddendum  "-4"
-#define MagickppLibInterface  7
-#define MagickppLibMinInterface  7
-#define MagickReleaseDate  "2017-01-14"
+#define MagickppLibVersionText  "6.9.10"
+#define MagickppLibVersionNumber  8:0:0
+#define MagickppLibAddendum  "-8"
+#define MagickppLibInterface  8
+#define MagickppLibMinInterface  8
+#define MagickReleaseDate  "2018-07-23"
 #define MagickHomeURL  "file:///usr/share/doc/ImageMagick-6/index.html"
 #define MagickAuthoritativeLicense  \
-  "http://www.imagemagick.org/script/license.php"
-#define MagickAuthoritativeURL  "http://www.imagemagick.org"
+  "https://www.imagemagick.org/script/license.php"
+#define MagickAuthoritativeURL  "https://www.imagemagick.org"
 #if (MAGICKCORE_QUANTUM_DEPTH == 8)
 #define MagickQuantumDepth  "Q8"
 #define MagickQuantumRange  "255"
