@@ -18,13 +18,13 @@
 %                               March 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -48,6 +48,7 @@
 #include <locale.h>
 #include "wand/MagickWand.h"
 #include "magick/colorspace-private.h"
+#include "magick/log.h"
 #include "magick/resource_.h"
 #include "magick/string-private.h"
 #include "validate.h"
@@ -1164,6 +1165,8 @@ static size_t ValidateConvertCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
           GetMagickModule());
+        (void) LogMagickEvent(ExceptionEvent,GetMagickModule(),
+          "%s",exception->reason);
         (*fail)++;
         continue;
       }
@@ -1378,8 +1381,8 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
       (void) CopyMagickString(image_info->filename,reference_filename,
         MaxTextExtent);
       reference_image=ReadImage(image_info,exception);
-      if (reference_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+      if ((reference_image == (Image *) NULL) ||
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1398,7 +1401,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
         reference_formats[i].magick,output_filename);
       status=SetImageType(reference_image,reference_types[j].type);
       InheritException(exception,&reference_image->exception);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1409,7 +1412,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
         }
       status=SetImageDepth(reference_image,reference_types[j].depth);
       InheritException(exception,&reference_image->exception);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1422,10 +1425,12 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
       status=WriteImage(image_info,reference_image);
       InheritException(exception,&reference_image->exception);
       reference_image=DestroyImage(reference_image);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
+          if (exception->reason != (char *) NULL)
+            (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
           (*fail)++;
           continue;
@@ -1437,10 +1442,12 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
         reference_formats[i].magick,output_filename);
       ping_image=PingImage(image_info,exception);
       if (ping_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
+          if (exception->reason != (char *) NULL)
+            (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
           (*fail)++;
           continue;
@@ -1452,11 +1459,13 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
       (void) FormatLocaleString(image_info->filename,MaxTextExtent,"%s:%s",
         reference_formats[i].magick,output_filename);
       reference_image=ReadImage(image_info,exception);
-      if (reference_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+      if ((reference_image == (Image *) NULL) ||
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
+          if (exception->reason != (char *) NULL)
+            (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
           (*fail)++;
           continue;
@@ -1472,11 +1481,13 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
       reference_image->compression=reference_formats[i].compression;
       length=8192;
       blob=ImageToBlob(image_info,reference_image,&length,exception);
-      if (blob == (unsigned char *) NULL ||
-          exception->severity >= ErrorException)
+      if ((blob == (unsigned char *) NULL) ||
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
+          if (exception->reason != (char *) NULL)
+            (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
           (*fail)++;
           reference_image=DestroyImage(reference_image);
@@ -1487,10 +1498,12 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
       */
       ping_image=PingBlob(image_info,blob,length,exception);
       if (ping_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
+          if (exception->reason != (char *) NULL)
+            (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
           (*fail)++;
           blob=(unsigned char *) RelinquishMagickMemory(blob);
@@ -1505,10 +1518,12 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
       reconstruct_image=BlobToImage(image_info,blob,length,exception);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       if (reconstruct_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
+          if (exception->reason != (char *) NULL)
+            (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
           (*fail)++;
           reference_image=DestroyImage(reference_image);
@@ -1525,10 +1540,12 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
       reconstruct_image=DestroyImage(reconstruct_image);
       reference_image=DestroyImage(reference_image);
       if (difference_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
+          if (exception->reason != (char *) NULL)
+            (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
           (*fail)++;
           continue;
@@ -1647,8 +1664,8 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
       (void) CopyMagickString(image_info->filename,reference_filename,
         MaxTextExtent);
       reference_image=ReadImage(image_info,exception);
-      if (reference_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+      if ((reference_image == (Image *) NULL) ||
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1667,7 +1684,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
         reference_formats[i].magick,output_filename);
       status=SetImageType(reference_image,reference_types[j].type);
       InheritException(exception,&reference_image->exception);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1678,7 +1695,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
         }
       status=SetImageDepth(reference_image,reference_types[j].depth);
       InheritException(exception,&reference_image->exception);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1691,7 +1708,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
       status=WriteImage(image_info,reference_image);
       InheritException(exception,&reference_image->exception);
       reference_image=DestroyImage(reference_image);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1705,8 +1722,8 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
       (void) FormatLocaleString(image_info->filename,MaxTextExtent,"%s:%s",
         reference_formats[i].magick,output_filename);
       reference_image=ReadImage(image_info,exception);
-      if (reference_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+      if ((reference_image == (Image *) NULL) ||
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1723,7 +1740,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
       reference_image->compression=reference_formats[i].compression;
       status=WriteImage(image_info,reference_image);
       InheritException(exception,&reference_image->exception);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1739,7 +1756,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
         reference_formats[i].magick,output_filename);
       reconstruct_image=ReadImage(image_info,exception);
       if (reconstruct_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1759,7 +1776,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
       reconstruct_image=DestroyImage(reconstruct_image);
       reference_image=DestroyImage(reference_image);
       if (difference_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1863,8 +1880,8 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
       (void) CopyMagickString(image_info->filename,reference_filename,
         MaxTextExtent);
       reference_image=ReadImage(image_info,exception);
-      if (reference_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+      if ((reference_image == (Image *) NULL) ||
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1881,7 +1898,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
         reference_image->rows*reference_storage[j].quantum;
       pixels=(unsigned char *) AcquireQuantumMemory(length,sizeof(*pixels));
       if (pixels == (unsigned char *) NULL ||
-          exception->severity >= ErrorException)
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1890,11 +1907,11 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
           reference_image=DestroyImage(reference_image);
           continue;
         }
-      (void) ResetMagickMemory(pixels,0,length*sizeof(*pixels));
+      (void) memset(pixels,0,length*sizeof(*pixels));
       status=ExportImagePixels(reference_image,0,0,reference_image->columns,
         reference_image->rows,reference_map[i],reference_storage[j].type,pixels,
         exception);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1909,7 +1926,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
         reference_image->rows,reference_map[i],reference_storage[j].type,
         pixels);
       InheritException(exception,&reference_image->exception);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1932,7 +1949,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
         pixels);
       InheritException(exception,&reconstruct_image->exception);
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-      if (status == MagickFalse || exception->severity >= ErrorException)
+      if (status == MagickFalse || (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
@@ -1949,7 +1966,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
       reconstruct_image=DestroyImage(reconstruct_image);
       reference_image=DestroyImage(reference_image);
       if (difference_image == (Image *) NULL ||
-          exception->severity >= ErrorException)
+          (exception->severity >= ErrorException))
         {
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
