@@ -6,6 +6,7 @@
 #
 # Package base name
 PACKAGE_NAME='ImageMagick'
+PACKAGE_BUGREPORT="https://github.com/ImageMagick/ImageMagick/issues"
 
 #
 # Date of last ChangeLog update
@@ -15,11 +16,11 @@ PACKAGE_CHANGE_DATE=`awk '/^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]/ { print
 #
 # Package version.  This is is the numeric version suffix applied to
 # PACKAGE_NAME (e.g. "1.0.0").
-PACKAGE_VERSION='6.9.7'
-PACKAGE_PERL_VERSION='6.97'
-PACKAGE_LIB_VERSION="0x697"
-PACKAGE_RELEASE="4"
-PACKAGE_LIB_VERSION_NUMBER="6,9,7,${PACKAGE_RELEASE}"
+PACKAGE_VERSION='6.9.10'
+PACKAGE_PERL_VERSION='6.9.10'
+PACKAGE_LIB_VERSION="0x69A"
+PACKAGE_RELEASE="8"
+PACKAGE_LIB_VERSION_NUMBER="6,9,10,${PACKAGE_RELEASE}"
 PACKAGE_RELEASE_DATE_RAW=`date +%F`
 PACKAGE_RELEASE_DATE_REPRODUCIBLE="${PACKAGE_CHANGE_DATE}"
 PACKAGE_STRING="$PACKAGE_NAME $PACKAGE_VERSION"
@@ -40,11 +41,11 @@ PACKAGE_VERSION_ADDENDUM="-${PACKAGE_RELEASE}"
 # PLEASE NOTE that doing a SO BUMP aka raising the CURRENT REVISION
 # could be avoided using libversioning aka map files.  You MUST change .map
 # files if you raise these versions.
-MAGICK_LIBRARY_CURRENT=3
+MAGICK_LIBRARY_CURRENT=6
 MAGICK_LIBRARY_REVISION=0
 MAGICK_LIBRARY_AGE=0
 
 # magick++
-MAGICKPP_LIBRARY_CURRENT=7
+MAGICKPP_LIBRARY_CURRENT=8
 MAGICKPP_LIBRARY_REVISION=0
 MAGICKPP_LIBRARY_AGE=0
