@@ -22,13 +22,13 @@
 %                                March 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -131,7 +131,7 @@ WandExport WandView *CloneWandView(const WandView *wand_view)
   if (clone_view == (WandView *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       wand_view->name);
-  (void) ResetMagickMemory(clone_view,0,sizeof(*clone_view));
+  (void) memset(clone_view,0,sizeof(*clone_view));
   clone_view->id=AcquireWandId();
   (void) FormatLocaleString(clone_view->name,MaxTextExtent,"%s-%.20g",
     WandViewId,(double) clone_view->id);
@@ -293,8 +293,8 @@ WandExport MagickBooleanType DuplexTransferWandViewIterator(WandView *source,
   exception=destination->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   height=(size_t) (source->extent.height-source->extent.y);
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(source_image,destination_image,height,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(source_image,destination_image,height,1)
 #endif
   for (y=source->extent.y; y < (ssize_t) source->extent.height; y++)
   {
@@ -571,8 +571,8 @@ WandExport MagickBooleanType GetWandViewIterator(WandView *source,
   progress=0;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   height=(size_t) (source->extent.height-source->extent.y);
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(source_image,source_image,height,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(source_image,source_image,height,1)
 #endif
   for (y=source->extent.y; y < (ssize_t) source->extent.height; y++)
   {
@@ -763,7 +763,7 @@ static PixelWand ***AcquirePixelsThreadSet(const size_t number_wands,
     sizeof(*pixel_wands));
   if (pixel_wands == (PixelWand ***) NULL)
     return((PixelWand ***) NULL);
-  (void) ResetMagickMemory(pixel_wands,0,number_threads*sizeof(*pixel_wands));
+  (void) memset(pixel_wands,0,number_threads*sizeof(*pixel_wands));
   for (i=0; i < (ssize_t) number_threads; i++)
   {
     pixel_wands[i]=NewPixelWands(number_wands);
@@ -784,7 +784,7 @@ WandExport WandView *NewWandView(MagickWand *wand)
   if (wand_view == (WandView *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       GetExceptionMessage(errno));
-  (void) ResetMagickMemory(wand_view,0,sizeof(*wand_view));
+  (void) memset(wand_view,0,sizeof(*wand_view));
   wand_view->id=AcquireWandId();
   (void) FormatLocaleString(wand_view->name,MaxTextExtent,"%s-%.20g",
     WandViewId,(double) wand_view->id);
@@ -845,7 +845,7 @@ WandExport WandView *NewWandViewExtent(MagickWand *wand,const ssize_t x,
   if (wand_view == (WandView *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       GetExceptionMessage(errno));
-  (void) ResetMagickMemory(wand_view,0,sizeof(*wand_view));
+  (void) memset(wand_view,0,sizeof(*wand_view));
   wand_view->id=AcquireWandId();
   (void) FormatLocaleString(wand_view->name,MaxTextExtent,"%s-%.20g",
     WandViewId,(double) wand_view->id);
@@ -980,8 +980,8 @@ WandExport MagickBooleanType SetWandViewIterator(WandView *destination,
   exception=destination->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   height=(size_t) (destination->extent.height-destination->extent.y);
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(destination_image,destination_image,height,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(destination_image,destination_image,height,1)
 #endif
   for (y=destination->extent.y; y < (ssize_t) destination->extent.height; y++)
   {
@@ -1073,7 +1073,7 @@ MagickExport void SetWandViewThreads(WandView *image_view,
   const size_t number_threads)
 {
   assert(image_view != (WandView *) NULL);
-  assert(image_view->signature == MagickSignature);
+  assert(image_view->signature == MagickCoreSignature);
   image_view->number_threads=number_threads;
   if (number_threads > (size_t) GetMagickResourceLimit(ThreadResource))
     image_view->number_threads=GetOpenMPMaximumThreads();
@@ -1164,8 +1164,8 @@ WandExport MagickBooleanType TransferWandViewIterator(WandView *source,
   exception=destination->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   height=(size_t) (source->extent.height-source->extent.y);
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(source_image,destination_image,height,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(source_image,destination_image,height,1)
 #endif
   for (y=source->extent.y; y < (ssize_t) source->extent.height; y++)
   {
@@ -1339,8 +1339,8 @@ WandExport MagickBooleanType UpdateWandViewIterator(WandView *source,
   exception=source->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   height=(size_t) (source->extent.height-source->extent.y);
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(source_image,source_image,height,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(source_image,source_image,height,1)
 #endif
   for (y=source->extent.y; y < (ssize_t) source->extent.height; y++)
   {
